{Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved.}

{TERATERM.EXE, Communication routines}
unit COMMLIB;

interface
{$I teraterm.inc}

{$IFDEF Delphi}
uses Messages, WinTypes, WinProcs, OWindows, Strings,
     Kanji, TTTypes, TTLib, WinsockH, Winsock, TTCommon;
{$ELSE}
uses Win31, WinTypes, WinProcs, WObjects, Strings,
     Kanji, TTTypes, TTLib, WinsockH, Winsock, TTCommon;
{$ENDIF}

const
  ErrorCaption = 'Tera Term: Error';
  ErrorCantConn = 'Cannot connect the host';
  CommInQueSize = 8092;
  CommOutQueSize = 2048;
  CommXonLim = 2048;
  CommXoffLim = 2048;

procedure CommOpen(HW: HWnd; ts: PTTSet; cv: PComVar);
procedure CommStart(cv: PComVar; lParam: longint);
procedure CommClose(cv: PComVar);
procedure CommProcRRQ(cv: PComVar);
procedure CommProcOOB(cv: PComVar);
procedure CommReceive(cv: PComVar);
procedure CommSend(cv: PComVar);
procedure CommSendBreak(cv: PComVar);
procedure CommResetSerial(ts: PTTSet; cv: PComVar);

implementation

{ reset a serial port which is already open }
procedure CommResetSerial(ts: PTTSet; cv: PComVar);
var
  dcb: TDCB;
  b: byte;
  Stat: TComStat;
begin

  if not cv^.Open or
     (cv^.PortType <> IdSerial) then exit;

  while GetCommError(cv^.ComId, Stat)<>0 do ;
  {flush input & output que}

  FlushComm(cv^.ComId,0);
  FlushComm(cv^.ComId,1);
  cv^.InBuffCount := 0;
  cv^.InPtr := 0;
  cv^.OutBuffCount := 0;
  cv^.OutPtr := 0;

  cv^.DelayPerChar := ts^.DelayPerChar;
  cv^.DelayPerLine := ts^.DelayPerLine;

  GetCommState(cv^.ComId,dcb);
  b := dcb.Id;
  fillchar(dcb,sizeof(dcb),0);
  dcb.Id := b;
  case  ts^.Baud of
    IdBaud110: dcb.BaudRate := 110;
    IdBaud300: dcb.BaudRate := 300;
    IdBaud600: dcb.BaudRate := 600;
    IdBaud1200: dcb.BaudRate := 1200;
    IdBaud2400: dcb.BaudRate := 2400;
    IdBaud4800: dcb.BaudRate := 4800;
    IdBaud9600: dcb.BaudRate := 9600;
    IdBaud19200: dcb.BaudRate := 19200;
    IdBaud38400: dcb.BaudRate := 38400;
    IdBaud57600: dcb.BaudRate := 57600;
    {IdBaud115200: dcb.BaudRate := 115200;}
  end;
  dcb.Flags := dcb.Flags or dcb_Binary;
  case ts^.Parity of
    IdParityEven: begin
        dcb.Flags := dcb.Flags or dcb_Parity;
        dcb.Parity := EVENPARITY;
      end;
    IdParityOdd: begin
        dcb.Flags := dcb.Flags or dcb_Parity;
        dcb.Parity := ODDPARITY;
      end;
    IdParityNone:
      dcb.Parity := NOPARITY;
  end;

  case ts^.Flow of
    IdFlowX: begin
        dcb.Flags := dcb.Flags or dcb_OutX;
        dcb.Flags := dcb.Flags or dcb_InX;
        dcb.XonLim := CommXonLim;
        dcb.XoffLim := CommXoffLim;
        dcb.XonChar := char(XON);
        dcb.XoffChar := char(XOFF);
      end;
    IdFlowHard: begin
        dcb.CtsTimeOut := 30;
        dcb.Flags := dcb.Flags or dcb_OutxCtsFlow;
        dcb.Flags := dcb.Flags or dcb_RtsFlow;
      end;
  end;

  case ts^.DataBit of
    IdDataBit7: dcb.ByteSize := 7;
    IdDataBit8: dcb.ByteSize := 8;
  end;
  case ts^.StopBit of
    IdStopBit1: dcb.StopBits := ONESTOPBIT;
    IdStopBit2: dcb.StopBits := TWOSTOPBITS;
  end;
  SetCommState(dcb);

end;

procedure CommOpen(HW: HWnd; ts: PTTSet; cv: PComVar);
var
  COMFlag: byte;
  Err: integer;
  ErrMsg: array[0..20] of Char;
  P: array[0..49] of Char;

  HAsync: THandle;
  HEntBuff: array[0..MAXGETHOSTSTRUCT-1] of char;
  addr: u_long;
  saddr: sockaddr_in;

  InvalidHost: boolean;
  Msg: TMsg;

  BBuf: bool;
  ZERO: longint;
  writefds: Tfd_set;
  timeout: timeval;

  hSaveCursor: HCURSOR;
begin
with cv^ do begin

  {initialize ComVar}
  InBuffCount := 0;
  InPtr := 0;
  OutBuffCount := 0;
  OutPtr := 0;
  HWin := HW;
  Ready := false;
  Open := false;
  PortType := ts^.PortType;
  ComPort := 0;
  RetryCount := 0;
  s := INVALID_SOCKET;
  ComID := -1;
  CanSend := true;
  RRQ := false;
  SendKanjiFlag := false;
  SendCode := IdASCII;
  EchoKanjiFlag := false;
  EchoCode := IdASCII;
  Language := ts^.Language;
  CRSend := ts^.CRSend;
  KanjiCodeEcho := ts^.KanjiCode;
  JIS7KatakanaEcho := ts^.JIS7Katakana;
  KanjiCodeSend := ts^.KanjiCodeSend;
  JIS7KatakanaSend := ts^.JIS7KatakanaSend;
  KanjiIN := ts^.KanjiIN;
  KanjiOut := ts^.KanjiOut;
  DelayFlag := true;
  DelayPerChar := ts^.DelayPerChar;
  DelayPerLine := ts^.DelayPerLine;
  TelBinRecv := false;
  TelBinSend := false;
  TelFlag := false;
  TelMode := false;
  IACFlag := false;
  TelCRFlag := false;
  TelCRSend := false;
  TelCRSendEcho := false;
  HLogBuf := 0;
  HBinBuf := 0;
  LogBuf := nil;
  BinBuf := nil;
  LogPtr := 0;
  LStart := 0;
  LCount := 0;
  BinPtr := 0;
  BStart := 0;
  BCount := 0;
  DStart := 0;
  DCount := 0;
  BinSkip := 0;

  if (ts^.PortType<>IdSerial) and (StrLen(ts^.HostName)=0) then
  begin
    PostMessage(HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
    exit;
  end;

  case ts^.PortType of
    IdTCPIP:
    begin
      if not LoadWinsock then
      begin
        MessageBox(HWin,'Cannot use winsock',ErrorCaption,
                   mb_TaskModal or mb_IconExclamation);
        InvalidHost := true;
      end
      else begin
        Open := true;
        if (ts^.HostName[0] >= #$30) and (ts^.HostName[0] <= #$39)
        then begin
          addr := inet_addr(ts^.HostName);
          InvalidHost := addr = $ffffffff;
        end
        else begin
          HAsync := WSAAsyncGetHostByName(HWin,WM_USER+MsgGetHost,
            ts^.HostName,HEntBuff,MAXGETHOSTSTRUCT);
          if HAsync=0 then
            InvalidHost := TRUE
          else begin
            repeat {loop until the host address is retrieved}
             if GetMessage(Msg,0,0,0) then
              begin
                if (Msg.message = WM_SYSCOMMAND) and
                   (Msg.wParam = SC_CLOSE) or
		   (Msg.message = WM_COMMAND) and
		   (LOWORD(Msg.wParam) = CM_EXIT) or
		   (Msg.message = WM_CLOSE) then
                begin {Exit when the user closes Tera Term}
		  WSACancelAsyncRequest(HAsync);
                  PostMessage(HWin,Msg.message,Msg.wParam,Msg.lParam);
                  exit;
                end;
                if not (Msg.message=WM_USER+MsgGetHost) then
                begin {Prosess messages}
                  TranslateMessage(Msg);
                  DispatchMessage(Msg);
                end;
              end
              else begin
                PostMessage(HWin,wm_First+wm_Quit,0,0);
                exit;
              end;
            until Msg.message=WM_USER+MsgGetHost;

            InvalidHost := WSAGETASYNCERROR(Msg.lParam) <> 0;
            if not InvalidHost then
            begin
              if PHostEnt(@HEntBuff[0])^.h_addr_list <> nil then
                move(PHostEnt(@HEntBuff[0])^.h_addr_list^[0],addr,SizeOf(addr))
              else
                InvalidHost := TRUE;
            end;
          end;
        end;

        if InvalidHost then
          MessageBox(HWin,'Invalid host',ErrorCaption,
                     mb_TaskModal or mb_IconExclamation)
        else begin
          s:= _socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
          if s=INVALID_SOCKET
          then
            MessageBox(HWin,ErrorCantConn,ErrorCaption,
                       mb_TaskModal or mb_IconExclamation)
          else begin
            BBuf := true;
            setsockopt(s,SOL_SOCKET,SO_OOBINLINE,@BBuf,sizeof(BBuf));

            WSAAsyncSelect(s,HWin,WM_USER+MsgComOpen, FD_Connect);
            saddr.sin_family := AF_INET;
            saddr.sin_port := htons(ts^.TCPPort);
            saddr.sin_addr.s_addr := addr;
            fillchar(saddr.sin_zero[0],8,#0);

            Err := connect(s,SockAddr(saddr),SizeOf(saddr));
            if Err<>0 then Err := WSAGetLastError;
            if Err=WSAEWOULDBLOCK then
            begin
              {Do nothing}
            end
            else if Err<>0 then
              PostMessage(HWin, WM_USER+MsgComOpen,0,
                          MAKELONG(FD_CONNECT,Err));
          end;
        end;
      end;
    end;

    IdSerial:
    begin
      StrCopy(@P,'COM');
      P[3] := char($30+ts^.ComPort);
      P[4] := #0;
      ComID := OpenComm(@P,CommInQueSize,CommOutQueSize);
      if ComID < 0 then
      begin
        StrCopy(@ErrMsg,'Cannot open ');
        StrCat(@ErrMsg,@P);
        MessageBox(HWin,@ErrMsg,ErrorCaption,
                   mb_TaskModal or mb_IconExclamation);
        InvalidHost := true;
      end
      else begin
        Open := true;
        CommResetSerial(ts,cv);

        {notify to VT window that Comm Port is open}
        PostMessage(HWin, WM_USER+MsgComOpen, 0, 0);

        {disable comm notification}
        EnableCommNotification(ComId,0,-1,-1);
        InvalidHost := false;

        COMFlag := GetCOMFlag;
        case ts^.ComPort of
          1: ComFlag := ComFlag or 1;
          2: ComFlag := ComFlag or 2;
          3: ComFlag := ComFlag or 4;
          4: ComFlag := ComFlag or 8;
        end;
        SetComFlag(ComFlag);
      end;
    end;

    IdFile:
    begin
      StrUpperJ(ts^.HostName);
      ComId := _lopen(ts^.HostName,0);
      InvalidHost := ComId<=0;
      if InvalidHost then
        MessageBox(HWin,'Cannot open file',ErrorCaption,
                   mb_TaskModal or mb_IconExclamation)
      else begin
        Open := true;
        PostMessage(HWin, WM_USER+MsgComOpen, 0, 0);
      end;
    end;

  end;

  if InvalidHost then
  begin
    PostMessage(HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
    if (ts^.PortType=IdTCPIP) and Open then
    begin
      if S<>INVALID_SOCKET then closesocket(s);
      FreeWinsock;
    end;
    exit;
  end;

  case PortType of
    IdTCPIP: TelFlag := ts^.Telnet > 0;
    IdSerial: ComPort := ts^.ComPort;
  end;

end;
end;

procedure CommStart(cv: PComVar; lParam: longint);
var
  ErrMsg: array[0..30] of char;
  Stat: TComStat;
  Err: integer;
begin
with cv^ do begin
  if not Open then exit;
  if Ready then exit;
  case PortType of
    IdTCPIP:
      begin
        ErrMsg[0] := #0;
        case HiWord(lParam) of
          WSAECONNREFUSED: StrCopy(ErrMsg,'Connection refused');
          WSAENETUNREACH: StrCopy(ErrMsg,'Network cannot be reached');
          WSAETIMEDOUT: StrCopy(ErrMsg,'Connection timed out');
        else
          StrCopy(ErrMsg,ErrorCantConn);
        end;
        if HiWord(lParam)>0 then
        begin
          MessageBox(HWin,ErrMsg,ErrorCaption,
                   mb_TaskModal or mb_IconExclamation);
          PostMessage(HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
          exit;
        end;
        WSAAsyncSelect(s,HWin,WM_USER+MsgComNotify, FD_Read or FD_OOB or FD_Close);
      end;
    IdSerial:
      begin
        {flush input que}
        while GetCommError(ComId, Stat)<>0 do ;
        FlushComm(ComId,1);
        {enable receive request}
        SetCommEventMask(ComId,EV_RXCHAR);
        EnableCommNotification(ComId,HWin,-1,-1);
        GetCommEventMask(ComId,EV_RXCHAR);
      end;
    IdFile: RRQ := true;
  end;
  Ready := true;
end;
end;

procedure CommClose(cv: PComVar);
var
  COMFlag: byte;
begin
with cv^ do begin
  if not Open then exit;
  {close port & release resources}
  {disable event message posting & flush buffer}
  
  if HLogBuf<>0 then
  begin
    if LogBuf<>nil then
      GlobalUnlock(HLogBuf);
    GlobalFree(HLogBuf);
  end;
  if HBinBuf<>0 then
  begin
    if BinBuf<>nil then
      GlobalUnlock(HBinBuf);
    GlobalFree(HBinBuf);
  end;
  HLogBuf := 0;
  LogBuf := nil; 
  LogPtr := 0;
  HBinBuf := 0;
  BinBuf := nil;
  BinPtr := 0;
  LStart := 0;
  LCount := 0;
  BStart := 0;
  BCount := 0;
  DStart := 0;
  DCount := 0;

  case PortType of
    IdTCPIP:
      begin
        if s<>INVALID_SOCKET then
          closesocket(s);
        s := INVALID_SOCKET;
        FreeWinsock;
      end;
    IdSerial: if ComId >= 0 then
      begin
        FlushComm(ComId,0);
        FlushComm(ComId,1);
        EscapeCommFunction(ComId,CLRDTR);
        EnableCommNotification(ComId,0,-1,-1) ;
        CloseComm(ComId);
        case ComPort of
          1: COMFlag := COMFlag and $0E;
          2: COMFlag := COMFlag and $0D;
          3: COMFlag := COMFlag and $0B;
          4: COMFlag := COMFlag and $07;
        else
          COMFlag := 0;
        end;
        SetCOMFlag(COMFlag);
        ComId := -1;
      end;
    IdFile: if ComId > 0 then
      begin
        _lclose(ComId);
        ComId := -1;
      end;
  end;
  InPtr := 0;
  InBuffCount := 0;
  OutPtr := 0;
  OutBuffCount := 0;
  RRQ := false;
  Ready := false;
  Open := false;
  PortType := 0;
end;
end;

procedure CommProcRRQ(cv: PComVar);
var
  Stat: TComStat;
  b: longint;
begin
with cv^ do begin
  if not Ready then exit;
  {disable receive request}
  case PortType of
    IdTCPIP:
      WSAAsyncSelect(s,HWin,WM_USER+MsgComNotify, FD_OOB or FD_Close);
    IdSerial:
      begin
        EnableCommNotification(ComId,0,-1,-1);
        while GetCommError(ComId, Stat)<>0 do ;
      end;
  end;
  RRQ := true;
  CommReceive(cv);
end;
end;

procedure CommProcOOB(cv: PComVar);
begin
with cv^ do begin
  if not Ready then exit;
end;
end;

procedure CommReceive(cv: PComVar);
var
  C: integer;
  Stat: TComStat;
begin
with cv^ do begin
  if not Ready or not RRQ or
     (InBuffCount>=InBuffSize) then exit;

  {Compact buffer}
  if (InBuffCount>0) and (InPtr>0) then
  begin
    Move(InBuff[InPtr],InBuff[0],InBuffCount);
    InPtr := 0;
  end;

  if InBuffCount<InBuffSize then
  begin
    case PortType of
      IdTCPIP:
        begin
          C := recv(s, @InBuff[InBuffCount], InBuffSize-InBuffCount, 0);
          if C < 0  then
          begin
            C := 0;
            WSAGetLastError;
          end;
          InBuffCount := InBuffCount + C;
        end;
      IdSerial:
          repeat
            C := ReadComm(ComId, @InBuff[InBuffCount], InBuffSize-InBuffCount);
            C := abs(C);
            while GetCommError(ComId, Stat)<>0 do ;
            InBuffCount := InBuffCount + C;
          until (C=0) or (InBuffCount>=InBuffSize);
      IdFile:
        begin
          C := _lread(ComId, @InBuff[InBuffCount], InBuffSize-InBuffCount);
          InBuffCount := InBuffCount + C;
        end;
    end;
  end;

  if InBuffCount=0 then
  begin
    case PortType of
      IdTCPIP:
        WSAAsyncSelect(s,HWin,WM_USER+MsgComNotify, FD_Read or FD_OOB or FD_Close);
      IdSerial:
        begin
          while GetCommError(ComId, Stat)<>0 do ;
          EnableCommNotification(ComId,HWin,-1,-1);
          GetCommEventMask(ComId,EV_RXCHAR);
        end;
      IdFile: PostMessage(HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
    end;
    RRQ := false;
  end;

end;
end;

procedure CommSend(cv: PComVar);
var
  C, D, Max, delay: integer;
  Stat: TComStat;
  LineEnd: byte;
begin
with cv^ do begin
  if not Ready then exit;

  if (OutBuffCount = 0) or (not CanSend) then exit;

  {Max num of bytes to be written}
  case PortType of
    IdTCPIP: Max := OutBuffCount; {winsock}
    IdSerial: begin
        GetCommError(ComId,Stat);
        Max := OutBuffSize - Stat.cbOutQue;
      end;
    IdFile: Max := OutBuffCount;
  end;

  if Max=0 then exit;
  if Max > OutBuffCount then Max := OutBuffCount;

  C := Max;
  delay := 0;

  if DelayFlag and (PortType=IdSerial) then
  begin
    if DelayPerLine > 0 then
    begin
      if CRSend=IdCR then LineEnd := $0d
                     else LineEnd := $0a;
      C := 1;
      if DelayPerChar=0 then
        while (C<Max) and (OutBuff[OutPtr+C-1]<>LineEnd) do
          inc(C);
      if OutBuff[OutPtr+C-1]=LineEnd then delay := DelayPerLine
                                     else delay := DelayPerChar;
    end
    else if DelayPerChar > 0 then
    begin
      C := 1;
      delay := DelayPerChar;
    end;
  end;

  {Write to comm driver/Winsock}
  case PortType of
    IdTCPIP:
      begin
        D := send(s, @OutBuff[OutPtr], C, 0);
        if D=SOCKET_ERROR then {if error occurs}
        begin
          WSAGetLastError; {Clear error}
          D := 0;
        end;
      end;
    IdSerial: begin
        D := WriteComm(ComId, @OutBuff[OutPtr], C);
        D := abs(D);
        while GetCommError(ComId, Stat)<>0 do ;
      end;
    IdFile: D := C;
  end;

  OutBuffCount := OutBuffCount - D;
  if OutBuffCount=0 then
    OutPtr := 0
  else
    OutPtr := OutPtr + D;

  if (C=D) and (delay>0) then
  begin
    CanSend := false;
    SetTimer(HWin, IdDelayTimer, delay, nil);
  end

end;
end;

procedure CommSendBreak(cv: PComVar);
var
  DummyMsg: TMsg;
begin
with cv^ do begin
  if not Ready then exit;

  if PortType<>IdSerial then exit; {if TCP port, can't send}

  {Set com port into a break state}
  SetCommBreak(ComId);

  {pause for 1sec}
  if SetTimer(HWin, IdBreakTimer, 1000, nil) <> 0 then
    GetMessage(DummyMsg,HWin,wm_First+wm_Timer,wm_First+wm_Timer);

  {Set com port into a nonbreak state}
  ClearCommBreak(ComId);
end;
end;

end.