{Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved.}

{TTCMN.DLL, main}
library TTCMN;
{$R teraterm.res}
{$I teraterm.inc}

{$IFDEF Delphi}
uses Messages, WinTypes, WinProcs, OWindows, Strings, WinDos, TTTypes, TTSetup, TTLib, Kanji;
{$ELSE}
uses Win31, WinTypes, WinProcs, WObjects, Strings, WinDos, TTTypes, TTSetup, TTLib, Kanji;
{$ENDIF}

const
  CommInQueSize = 8092;
  CommOutQueSize = 2048;

var
  {first instance flag}
  FirstInstance: bool;

  {Setup information from 'teraterm.ini'}
  tsdef: TTTSet;

  {Key code map from 'keyboard.cnf'}
  KeyMapDef: TKeyMap;

  { COM port use flag - bit0-3 : COM1-4 }
  ComFlag: byte;

procedure StartTeraTerm(ts: PTTSet); export;
var
  Temp: array[0..fsPathName] of char;
begin
  if FirstInstance then
  begin
    FirstInstance := false;
    {Get home directory}
    GetModuleFileName(HInstance,Temp,SizeOf(Temp));
    ExtractDirNameJ(Temp,tsdef.HomeDir);
    StrUpper(tsdef.HomeDir);
    SetCurDir(tsdef.HomeDir);
    StrCopy(tsdef.SetupFName,tsdef.HomeDir);
    StrCat(tsdef.SetupFName,'\TERATERM.INI');
    StrCopy(Temp,tsdef.HomeDir);
    StrCat(Temp,'\KEYBOARD.CNF');
    if LoadTTSET then
    begin
      {read setup info from 'teraterm.ini'}
      ReadIniFile(tsdef.SetupFName, @tsdef);
      {read keycode map from 'keyboard.cnf'}
      ReadKeyboardCnf(Temp,@KeyMapDef);
      FreeTTSET;
    end;
  end
  else begin
    {only the first instance uses saved position}
    tsdef.VTPos.x := CW_UseDefault;
    tsdef.VTPos.y := CW_UseDefault;
    tsdef.TEKPos.x := CW_UseDefault;
    tsdef.TEKPos.y := CW_UseDefault;
  end;

  ts^ := tsdef;
end;

procedure ChangeDefaultSet(ts: PTTSet); export;
begin
  if StrIComp(ts^.SetupFName,tsdef.SetupFName)<>0 then exit;

  tsdef := ts^;
end;

procedure LoadDefaultSet(SetupFName: PChar); export;
begin
  if StrIComp(SetupFName,tsdef.SetupFName)<>0 then
    exit;

  if LoadTTSET then
  begin
    {read setup info from 'teraterm.ini'}
    ReadIniFile(tsdef.SetupFName,@tsdef);
    FreeTTSET;
  end;
end;

{Key scan code -> Tera Term key code}
function GetKeyCode(KeyMap: PKeyMap; Scan: word): word; export;
var
  Key: word;
begin                                                   
  if KeyMap=nil then
    KeyMap := @KeyMapDef;
  Key := IdKeyMax;
  while (Key>0) and (KeyMap^.Map[Key-1]<>Scan) do
    Dec(Key);
  GetKeyCode := Key;
end;

procedure GetKeyStr(HWin: HWnd; KeyMap: PKeyMap; KeyCode: word; AppliKeyMode, AppliCursorMode: BOOL;
                    KeyStr: PChar; Len: Pinteger; Conv: PBOOL); export;
var
  i, j: integer;
  Msg: TMsg;
begin
  if KeyMap=nil then
    KeyMap := @KeyMapDef;

  Conv^ := true;
  Len^ := 0;
  case KeyCode of
      IdUp:
        begin
          Len^ := 3;
          if AppliCursorMode then StrCopy(KeyStr,#$1B'OA')
                             else StrCopy(KeyStr,#$1B'[A');
        end;
      IdDown:
        begin
          Len^ := 3;
          if AppliCursorMode then StrCopy(KeyStr,#$1B'OB')
                             else StrCopy(KeyStr,#$1B'[B');
        end;
      IdRight:
        begin
          Len^ := 3;
          if AppliCursorMode then StrCopy(KeyStr,#$1B'OC')
                             else StrCopy(KeyStr,#$1B'[C');
        end;
      IdLeft:
        begin
          Len^ := 3;
          if AppliCursorMode then StrCopy(KeyStr,#$1B'OD')
                             else StrCopy(KeyStr,#$1B'[D');
        end;
      Id0:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Op');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '0';
          end;
        end;
      Id1:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Oq');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '1';
          end;
        end;
      Id2:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Or');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '2';
          end;
        end;
      Id3:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Os');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '3';
          end;
        end;
      Id4:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Ot');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '4';
          end;
        end;
      Id5:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Ou');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '5';
          end;
        end;
      Id6:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Ov');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '6';
          end;
        end;
      Id7:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Ow');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '7';
          end;
        end;
      Id8:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Ox');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '8';
          end;
        end;
      Id9:
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Oy');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '9';
          end;
        end;
      IdMinus: {numaric pad - key (DEC)}
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Om');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '-';
          end;
        end;
      IdComma: {numaric pad , key (DEC)}
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'Ol');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := ',';
          end;
        end;
      IdPeriod: {numaric pad . key}
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'On');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := '.';
          end;
        end;
      IdEnter: {numaric pad enter key}
        begin
          if AppliKeyMode
          then begin
            Len^ := 3;
            StrCopy(KeyStr,#$1B'OM');
          end
          else begin
            Len^ := 1;
            KeyStr[0] := #$0D;
          end;
        end;
      IdPF1: {DEC Key: PF1}
        begin
          Len^ := 3;
          StrCopy(KeyStr,#$1B'OP');
        end;
      IdPF2: {DEC Key: PF2}
        begin
          Len^ := 3;
          StrCopy(KeyStr,#$1B'OQ');
        end;
      IdPF3: {DEC Key: PF3}
        begin
          Len^ := 3;
          StrCopy(KeyStr,#$1B'OR');
        end;
      IdPF4: {DEC Key: PF4}
        begin
          Len^ := 3;
          StrCopy(KeyStr,#$1B'OS');
        end;
      IdFind: {DEC Key: Find}
        begin
          Len^ := 4;
          StrCopy(KeyStr,#$1B'[1~');
        end;
      IdInsert: {DEC Key: Insert Here}
        begin
          Len^ := 4;
          StrCopy(KeyStr,#$1B'[2~');
        end;
      IdRemove: {DEC Key: Remove}
        begin
          Len^ := 4;
          StrCopy(KeyStr,#$1B'[3~');
        end;
      IdSelect: {DEC Key: Select}
        begin
          Len^ := 4;
          StrCopy(KeyStr,#$1B'[4~');
        end;
      IdPrev: {DEC Key: Prev}
        begin
          Len^ := 4;
          StrCopy(KeyStr,#$1B'[5~');
        end;
      IdNext: {DEC Key: Next}
        begin
          Len^ := 4;
          StrCopy(KeyStr,#$1B'[6~');
        end;
      IdF6: {DEC Key: F6}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[17~');
        end;
      IdF7: {DEC Key: F7}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[18~');
        end;
      IdF8: {DEC Key: F8}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[19~');
        end;
      IdF9: {DEC Key: F9}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[20~');
        end;
      IdF10: {DEC Key: F10}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[21~');
        end;
      IdF11: {DEC Key: F11}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[23~');
        end;
      IdF12: {DEC Key: F12}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[24~');
        end;
      IdF13: {DEC Key: F13}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[25~');
        end;
      IdF14: {DEC Key: F14}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[26~');
        end;
      IdHelp: {DEC Key: Help}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[28~');
        end;
      IdDo: {DEC Key: Do}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[29~');
        end;
      IdF17: {DEC Key: F17}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[31~');
        end;
      IdF18: {DEC Key: F18}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[32~');
        end;
      IdF19: {DEC Key: F19}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[33~');
        end;
      IdF20: {DEC Key: F20}
        begin
          Len^ := 5;
          StrCopy(KeyStr,#$1B'[34~');
        end;
      IdAccEditCopy..
      IdAccSendBreak:
          Postmessage(hwin,wm_User+MsgAccelCommand,KeyCode,0);
      IdUser1..IdKeyMax:
        begin
          with KeyMap^ do begin
            if UserKeyConv[KeyCode-IdUser1]=0 then
              Conv^ := FALSE
            else
              Conv^ := TRUE;
            Len^ := UserKeyLen[KeyCode-IdUser1];
            Move(UserKeyStr[UserKeyPtr[KeyCode-IdUser1]],KeyStr[0],Len^);
          end;
        end;
  else
    exit;
  end;
  {remove wm_char message for used keycode}
  PeekMessage(msg,hwin,wm_first+wm_char,wm_first+wm_char,pm_remove);
end;

procedure SetCOMFlag(Com: byte); export;
begin
  ComFlag := Com and $0F;
end;

function GetCOMFlag: byte; export;
begin
  GetComFlag := ComFlag and $0F;
end;

function CommReadRawByte(cv: PComVar; b: Pbyte): integer; export;
begin
with cv^ do begin
  if not Ready then
  begin
    CommReadRawByte := 0;
    exit;
  end;

  if InBuffCount>0 then
  begin
    b^ := InBuff[InPtr];
    inc(InPtr);
    dec(InBuffCount);
    CommReadRawByte := 1;
  end
  else
    CommReadRawByte := 0;

  if InBuffCount=0 then InPtr := 0;
end;
end;

procedure CommInsert1Byte(cv: PComVar; b:byte); export;
begin
with cv^ do begin
  if not Ready then exit;

  if InPtr = 0
  then
    Move(InBuff[0],InBuff[1],InBuffCount)
  else
    Dec(InPtr);
  InBuff[InPtr] := b;
  Inc(InBuffCount);

  if HBinBuf<>0 then inc(BinSkip);
end;
end;

procedure Log1Bin(cv: PComVar; b: byte);
begin
  with cv^ do begin
    if FilePause then exit;
    if BinSkip > 0 then
    begin
      dec(BinSkip);
      exit;
    end;
    PChar(BinBuf)[BinPtr] := char(b);
    inc(BinPtr);
    if BinPtr>=InBuffSize then BinPtr := BinPtr-InBuffSize;
    if BCount>=InBuffSize then
    begin
      BCount := InBuffSize;
      BStart := BinPtr;
    end
    else inc(BCount);
  end;
end;

function CommRead1Byte(cv: PComVar; b: Pbyte): integer; export;
var
  c: integer;
begin
with cv^ do begin
  if not Ready then
  begin
    CommRead1Byte := 0;
    exit;
  end;

  if TelMode then
    c := 0
  else
    c := CommReadRawByte(cv,b);

  if c=1 then
  begin
    if IACFlag then
    begin
      IACFlag := false;
      if b^<>$FF then
      begin
        TelMode := true;
        CommInsert1Byte(cv,b^);
        if HBinBuf<>0 then dec(BinSkip);
        c := 0;
      end;
    end
    else if TelCRFlag then
    begin
      TelCRFlag := false;
      if b^=0 then c := 0;
    end
    else if TelFlag and (b^=$FF) then
    begin
      IACFlag := true;
      c := 0;
    end
    else if TelFlag and not TelBinRecv and (b^=$0D) then
      TelCRFlag := true;      
  end;

  if (c=1) and (HBinBuf<>0) then
    Log1Bin(cv, b^);

  CommRead1Byte := c;
end;
end;

function CommRawOut(cv: PComVar; B: PChar; C: integer): integer; export;
var
  a: integer;
begin
with cv^ do begin
  if not Ready then
  begin
    CommRawOut := C;
    exit;
  end;

  if C > OutBuffSize-OutBuffCount then
    a := OutBuffSize-OutBuffCount
  else
    a := C;
  if OutPtr > 0 then
  begin
    Move(OutBuff[OutPtr],OutBuff[0],OutBuffCount);
    OutPtr := 0;
  end;
  Move(B[0],OutBuff[OutBuffCount],a);
  OutBuffCount := OutBuffCount + a;
  CommRawOut := a;

end;
end;

function CommBinaryOut(cv: PComVar; B: PChar; C: integer): integer; export;
var
  a, i, Len: integer;
  d: array[0..2] of byte;
begin
with cv^ do begin
  if not Ready then
  begin
    CommBinaryOut := C;
    exit;
  end;
  i := 0;
  a := 1;
  while (a>0) and (i<C) do
  begin
    Len := 0;

    d[Len] := byte(B[i]);
    inc(Len);

    if TelFlag and (B[i]=#$0D) and
       not TelBinSend then
    begin
      d[Len] := $00;
      inc(Len);
    end;

    if TelFlag and (B[i]=#$ff) then
    begin
      d[Len] := $ff;
      inc(Len);
    end;

    if OutBuffSize-OutBuffCount-Len >=0 then
    begin
      CommRawOut(cv,@d[0],Len);
      a := 1;
    end
    else
      a := 0;

    i := i + a;
  end;
  CommBinaryOut := i;
end;
end;

function CommTextOut(cv: PComVar; B: PChar; C: integer): integer; export;
var
  i, TempLen: integer;
  K: word;
  TempStr: array[0..10] of char;
  SendCodeNew: integer;
  Full, KanjiFlagNew: boolean;
begin
with cv^ do begin
  if not Ready then
  begin
    CommTextOut := C;
    exit;
  end;

  Full := false;
  i := 0;
  while not Full and (i < C) do
  begin
    TempLen := 0;
    SendCodeNew := SendCode;

    if SendKanjiFlag then
    begin
      KanjiFlagNew := false;
      SendCodeNew := IdKanji;

      K := SendKanjiFirst shl 8 + byte(B[i]);
      if KanjiCodeSend = IdEUC then K := SJIS2EUC(K)
      else if KanjiCodeSend <> IdSJIS then K := SJIS2JIS(K);

      TempStr[0] := char(Hi(K));
      TempStr[1] := char(Lo(K));
      TempLen := 2;
    end
    else

      if (Language=IdJapanese) and IsDBCSLeadByte(byte(B[i]))
      then begin
        KanjiFlagNew := true;
        SendKanjiFirst := byte(B[i]);
        SendCodeNew := IdKanji;

        if (SendCode<>IdKanji) and
           (KanjiCodeSend=IdJIS) then
        begin
          TempStr[0] := #$1B;
          TempStr[1] := '$';
          if KanjiIN = IdKanjiInB then
            TempStr[2] := 'B'
          else
            TempStr[2] := '@';
          TempLen := 3;
        end
        else TempLen := 0;

      end
      else begin
        KanjiFlagNew := false;

        if (SendCode=IdKanji) and
           (KanjiCodeSend=IdJIS) then
        begin
          TempStr[0] := #$1B;
          TempStr[1] := '(';
          case KanjiOut of
            IdKanjiOutJ: TempStr[2] := 'J';
            IdKanjiOutH: TempStr[2] := 'H';
          else
            TempStr[2] := 'B';
          end;
          TempLen := 3;
        end
        else TempLen := 0;

        if (Language=IdJapanese) and
           ($A0<byte(B[i])) and (byte(B[i])<$E0) then
        begin
          SendCodeNew := IdKatakana;
          if (SendCode<>IdKatakana) and
             (KanjiCodeSend=IdJIS) and
             (JIS7KatakanaSend=1) then
          begin
            TempStr[TempLen] := char(SO);
            inc(TempLen);
          end;
        end
        else begin
          SendCodeNew := IdASCII;
          if (SendCode=IdKatakana) and
             (KanjiCodeSend=IdJIS) and
             (JIS7KatakanaSend=1) then
          begin
            TempStr[TempLen] := char(SI);
            inc(TempLen);
          end;
        end;

        case byte(B[i]) of
          $0d: begin
            TempStr[TempLen] := #$0d;
            inc(TempLen);
            if CRSend=IdCRLF then
            begin
              TempStr[TempLen] := #$0a;
              inc(TempLen);
            end
            else if TelFlag and not TelBinSend then
            begin
              TempStr[TempLen] := #0;
              inc(TempLen);
            end;
          end;
          $a1..$e0: if (Language=IdJapanese) then
          begin {Katakana}
            if KanjiCodeSend=IdEUC then
            begin
              TempStr[TempLen] := #$8E;
              inc(TempLen);
            end;
            if (KanjiCodeSend=IdJIS) and
               (JIS7KatakanaSend=1) then
              TempStr[TempLen] := char(byte(B[i]) and $7f)
            else
              TempStr[TempLen] := B[i];
            inc(TempLen);
          end
          else begin
            TempStr[TempLen] := B[i];
            inc(TempLen);
          end;
        else
          begin
            TempStr[TempLen] := B[i];
            inc(TempLen);
            if TelFlag and (B[i]=#$FF) then
            begin
              TempStr[TempLen] := #$FF;
              inc(TempLen);
            end;
          end;
        end; {end of 'case of'}

      end; {if ISDBCSLeadByte..then ... else ... end}

    if TempLen = 0 then
    begin
      inc(i);
      SendCode := SendCodeNew;
      SendKanjiFlag := KanjiFlagNew;
    end
    else begin
      Full := OutBuffSize-OutBuffCount-TempLen < 0;
      if not Full then
      begin
        inc(i);
        SendCode := SendCodeNew;
        SendKanjiFlag := KanjiFlagNew;
        CommRawOut(cv,TempStr,TempLen);
      end;
    end;

  end;

  CommTextOut := i;

end;
end;

function CommBinaryEcho(cv: PComVar; B: PChar; C: integer): integer; export;
var
  a, i, Len: integer;
  d: array[0..2] of byte;
begin
with cv^ do begin
  if not Ready then
  begin
    CommBinaryEcho := C;
    exit;
  end;

  if (InPtr>0) and (InBuffCount>0) then
  begin
    Move(InBuff[InPtr],InBuff[0],InBuffCount);
    InPtr := 0;
  end;

  i := 0;
  a := 1;
  while (a>0) and (i<C) do
  begin
    Len := 0;

    d[Len] := byte(B[i]);
    inc(Len);

    if TelFlag and (B[i]=#$0D) and
       not TelBinSend then
    begin
      d[Len] := $00;
      inc(Len);
    end;

    if TelFlag and (B[i]=#$ff) then
    begin
      d[Len] := $ff;
      inc(Len);
    end;

    if InBuffSize-InBuffCount-Len >=0 then
    begin
      Move(d[0],InBuff[InBuffCount],Len);
      InBuffCount := InBuffCount + Len;
      a := 1;
    end
    else
      a := 0;
    i := i + a;
  end;
  CommBinaryEcho := i;

end;
end;

function CommTextEcho(cv: PComVar; B: PChar; C: integer): integer; export;
var
  i, TempLen: integer;
  K: word;
  TempStr: array[0..10] of char;
  EchoCodeNew: integer;
  Full, KanjiFlagNew: boolean;
begin
with cv^ do begin
  if not Ready then
  begin
    CommTextEcho := C;
    exit;
  end;

  if (InPtr>0) and (InBuffCount>0) then
  begin
    Move(InBuff[InPtr],InBuff[0],InBuffCount);
    InPtr := 0;
  end;

  Full := false;
  i := 0;
  while not Full and (i < C) do
  begin
    TempLen := 0;
    EchoCodeNew := EchoCode;

    if EchoKanjiFlag then
    begin
      KanjiFlagNew := false;
      EchoCodeNew := IdKanji;

      K := EchoKanjiFirst shl 8 + byte(B[i]);
      if KanjiCodeEcho = IdEUC then K := SJIS2EUC(K)
      else if KanjiCodeEcho <> IdSJIS then K := SJIS2JIS(K);

      TempStr[0] := char(Hi(K));
      TempStr[1] := char(Lo(K));
      TempLen := 2;
    end
    else

      if (Language=IdJapanese) and IsDBCSLeadByte(byte(B[i]))
      then begin
        KanjiFlagNew := true;
        EchoKanjiFirst := byte(B[i]);
        EchoCodeNew := IdKanji;

        if (EchoCode<>IdKanji) and
           (KanjiCodeEcho=IdJIS) then
        begin
          TempStr[0] := #$1B;
          TempStr[1] := '$';
          if KanjiIN = IdKanjiInB then
            TempStr[2] := 'B'
          else
            TempStr[2] := '@';
          TempLen := 3;
        end
        else TempLen := 0;

      end
      else begin
        KanjiFlagNew := false;

        if (EchoCode=IdKanji) and
           (KanjiCodeEcho=IdJIS) then
        begin
          TempStr[0] := #$1B;
          TempStr[1] := '(';
          case KanjiOut of
            IdKanjiOutJ: TempStr[2] := 'J';
            IdKanjiOutH: TempStr[2] := 'H';
          else
            TempStr[2] := 'B';
          end;
          TempLen := 3;
        end
        else TempLen := 0;

        if (Language=IdJapanese) and
           ($A0<byte(B[i])) and (byte(B[i])<$E0) then
        begin
          EchoCodeNew := IdKatakana;
          if (EchoCode<>IdKatakana) and
             (KanjiCodeEcho=IdJIS) and
             (JIS7KatakanaEcho=1) then
          begin
            TempStr[TempLen] := char(SO);
            inc(TempLen);
          end;
        end
        else begin
          EchoCodeNew := IdASCII;
          if (EchoCode=IdKatakana) and
             (KanjiCodeEcho=IdJIS) and
             (JIS7KatakanaEcho=1) then
          begin
            TempStr[TempLen] := char(SI);
            inc(TempLen);
          end;
        end;

        case byte(B[i]) of
          $0d: begin
            TempStr[TempLen] := #$0d;
            inc(TempLen);
            if CRSend=IdCRLF then
            begin
              TempStr[TempLen] := #$0a;
              inc(TempLen);
            end
            else if TelFlag and not TelBinSend then
            begin
              TempStr[TempLen] := #0;
              inc(TempLen);
            end;

          end;
          $a1..$e0: if (Language=IdJapanese) then
          begin {Katakana}
            if KanjiCodeEcho=IdEUC then
            begin
              TempStr[TempLen] := #$8E;
              inc(TempLen);
            end;
            if (KanjiCodeEcho=IdJIS) and
               (JIS7KatakanaEcho=1) then
              TempStr[TempLen] := char(byte(B[i]) and $7f)
            else
              TempStr[TempLen] := B[i];
            inc(TempLen);
          end
          else begin
            TempStr[TempLen] := B[i];
            inc(TempLen);
          end;
        else    
          begin
            TempStr[TempLen] := B[i];
            inc(TempLen);
            if TelFlag and (B[i]=#$FF) then
            begin
              TempStr[TempLen] := #$FF;
              inc(TempLen);
            end;
          end;
        end; {end of 'case of'}

      end; {if ISDBCSLeadByte..then ... else ... end}

    if TempLen = 0 then
    begin
      inc(i);
      EchoCode := EchoCodeNew;
      EchoKanjiFlag := KanjiFlagNew;
    end
    else begin
      Full := InBuffSize-InBuffCount-TempLen < 0;
      if not Full then
      begin
        inc(i);
        EchoCode := EchoCodeNew;
        EchoKanjiFlag := KanjiFlagNew;
        Move(TempStr[0],InBuff[InBuffCount],TempLen);
        InBuffCount := InBuffCount + TempLen;
      end;
    end;

  end;

  CommTextEcho := i;

end;
end;

exports
  StartTeraTerm    index 1,
  ChangeDefaultSet index 3,
  LoadDefaultSet   index 4,
  GetKeyCode       index 5,
  GetKeyStr        index 6,
  
  SetCOMFlag       index 10,
  GetCOMFlag       index 11,
  CommReadRawByte  index 18,
  CommRead1Byte    index 19,
  CommInsert1Byte  index 20,
  CommRawOut       index 21,
  CommBinaryOut    index 22,
  CommTextOut      index 23,
  CommBinaryEcho   index 24,
  CommTextEcho     index 25;

begin
  FirstInstance := true;
end.