{Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved.}

{TTSET.DLL, setup file routines}
library TTSet;

uses WinTypes, WinProcs, Strings, WinDos, TTTypes, TTLib;

const
  Section='Tera Term';

type
  TList = array[0..20] of PChar;
  PList = ^TList;
const
  TermList: array[0..9] of PChar = (
    'VT100','VT100J','VT101','VT102','VT102J','VT220J','VT282','VT320','VT382',nil);

  BaudList: array[0..10] of PChar = (
    '110','300','600','1200','2400','4800','9600',
    '19200','38400','57600',nil);

function str2id(List: PList; str: PCHAR; DefId: WORD): WORD;
var
  i: WORD;
begin
  i := 0;
  while ((List^[i] <> nil) and (stricomp(List^[i],str) <> 0)) do
    inc(i);

  if List^[i] = nil then
    i := DefId
  else
    inc(i);

  str2id := i;
end;

procedure id2str(List: PList; Id, DefId: WORD; str: PCHAR);
var
  i: integer;
begin
  if Id=0 then
    i := DefId - 1
  else begin
    i := 0;
    while ((List^[i] <> nil) and (i < Id-1)) do
      inc(i);
    if List^[i] = nil then
      i := DefId - 1;
  end; 
  strcopy(str,List^[i]);
end;


procedure GetNthString(Source: PChar; Nth, Size: integer; Dest: Pchar);
var
  i, j, k: integer;
  c: Char;
begin
  i := 1;
  j := 0;
  k := 0;
  repeat
    c := Source[j];
    if c=',' then inc(i);
    inc(j);
    if (i=Nth) and (c<>',') and (k<Size-1) then
    begin
      Dest[k] := c;
      inc(k);
    end
  until (i>Nth) or (c=#0);
  Dest[k] := #0;
end;

procedure GetNthNum(Source: PChar; Nth: integer; var Num: integer);
var
  T: array[0..5] of char;
  c: integer;
begin
  GetNthString(Source,Nth,SizeOf(T),@T[0]);
  Val(StrPas(@T[0]),Num,c);
  if c <> 0 then Num := 0;
end;

function GetOnOff(Section, Key, FName: PChar; Default: BOOL): Word;
var
  Temp: array[0..3] of char;
begin
  GetPrivateProfileString(Section,Key,'',
                          Temp,SizeOf(Temp),FName);
  if Default then
  begin
    if StrIComp(Temp,'off')=0 then
      GetOnOff := 0
    else
      GetOnOff := 1;
  end
  else begin
    if StrIComp(Temp,'on')=0 then
      GetOnOff := 1
    else
      GetOnOff := 0;
  end;
end;

procedure WriteOnOff(Section, Key, FName: PChar; Flag: word);
var
  Temp: array[0..3] of char;
begin
  if Flag>0 then StrCopy(Temp,'on')
            else StrCopy(Temp,'off');
  WritePrivateProfileString(Section,Key,Temp,FName);
end;

procedure ReadIniFile(FName: PChar; ts: PTTSet); export;
var
  i: integer;
  TmpDC: HDC;
  Temp: array[0..80] of char;
  AskUser: BOOL;
begin
  AskUser := false;
  {Version number}
  GetPrivateProfileString(Section,'Version','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'1.3')<>0 then
    AskUser := true;                     

  {Language}
  GetPrivateProfileString(Section,'Language','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'Japanese')=0 then
    ts^.Language := IdJapanese
  else if StrIComp(Temp,'English')=0 then
    ts^.Language := IdEnglish
  else begin
    ts^.Language := IdEnglish;
    AskUser := true;
  end;

  {Port type}
  GetPrivateProfileString(Section,'Port','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'tcpip')=0 then
    ts^.PortType := IdTCPIP
  else if StrIComp(Temp,'serial')=0 then
    ts^.PortType := IdSerial
  else begin
    ts^.PortType := IdSerial;
    AskUser := true;
  end;

  {VT win position}
  GetPrivateProfileString(Section,'VTPos','-32768,-32768',
                          Temp,SizeOf(Temp),FName);  {default: random position}
  GetNthNum(Temp,1,ts^.VTPos.x);
  GetNthNum(Temp,2,ts^.VTPos.y);

  if (ts^.VTPos.x < -20) or (ts^.VTPos.y < -20) then
  begin
    ts^.VTPos.x := CW_UseDefault;
    ts^.VTPos.y := CW_UseDefault;
  end
  else begin
    if ts^.VTPos.x < 0 then ts^.VTPos.x := 0;
    if ts^.VTPos.y < 0 then ts^.VTPos.y := 0;
  end;

  {TEK win position}
  GetPrivateProfileString(Section,'TEKPos','-32768,-32768',
                          Temp,SizeOf(Temp),FName);  {default: random position}
  GetNthNum(Temp,1,ts^.TEKPos.x);
  GetNthNum(Temp,2,ts^.TEKPos.y);

  if (ts^.TEKPos.x < -20) or (ts^.TEKPos.y < -20) then
  begin
    ts^.TEKPos.x := CW_UseDefault;
    ts^.TEKPos.y := CW_UseDefault;
  end
  else begin
    if ts^.TEKPos.x < 0 then ts^.TEKPos.x := 0;
    if ts^.TEKPos.y < 0 then ts^.TEKPos.y := 0;
  end;

  {VT terminal size }
  GetPrivateProfileString(Section,'TerminalSize','80,24',
                          Temp,SizeOf(Temp),FName);
  GetNthNum(Temp,1,ts^.TerminalWidth);
  GetNthNum(Temp,2,ts^.TerminalHeight);
  if ts^.TerminalWidth < 0 then ts^.TerminalWidth := 1;
  if ts^.TerminalHeight < 0 then ts^.TerminalHeight := 1;

  {Terminal size = Window size}
  ts^.TermIsWin := GetOnOff(Section,'TermIsWin',FName,false);

  {Auto window resize flag}
  ts^.AutoWinResize := GetOnOff(Section,'AutoWinResize',FName,false);

  {CR Receive}
  GetPrivateProfileString(Section,'CRReceive','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'CRLF')=0 then ts^.CRReceive := IdCRLF
                             else ts^.CRReceive := IDCR;
  {CR Send}
  GetPrivateProfileString(Section,'CRSend','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'CRLF')=0 then ts^.CRSend := IdCRLF
                             else ts^.CRSend := IdCR;

  {Local echo}
  ts^.LocalEcho := GetOnOff(Section,'LocalEcho',FName,false);

  {Auto B-Plus activation -- read-only option}
  ts^.BPAuto := GetOnOff(Section,'BPAuto',FName,false)>0;

  {Answerback}
  if not ts^.BPAuto then
  begin
    GetPrivateProfileString(Section,'Answerback','',Temp,
                            SizeOf(Temp),Fname);
    ts^.AnswerbackLen :=
      Hex2Str(Temp,ts^.Answerback,SizeOf(ts^.Answerback));
  end
  else begin
    StrCopy(ts^.Answerback,#$10'++'#$10'0');
    ts^.AnswerbackLen := 5;
  end;

  {Kanji Code (receive)}
  GetPrivateProfileString(Section,'KanjiReceive','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'EUC')=0 then ts^.KanjiCode := IdEUC
  else if StrIComp(Temp,'JIS')=0 then ts^.KanjiCode := IdJIS
  else ts^.KanjiCode := IdSJIS;

  {Katakana (receive)}
  GetPrivateProfileString(Section,'KatakanaReceive','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'7')=0 then ts^.JIS7Katakana := 1
  else ts^.JIS7Katakana := 0;


  {Kanji Code (transmit)}
  GetPrivateProfileString(Section,'KanjiSend','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'EUC')=0 then ts^.KanjiCodeSend := IdEUC
  else if StrIComp(Temp,'JIS')=0 then ts^.KanjiCodeSend := IdJIS
  else ts^.KanjiCodeSend := IdSJIS;

  {Katakana (receive)}
  GetPrivateProfileString(Section,'KatakanaSend','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'7')=0 then ts^.JIS7KatakanaSend := 1
  else ts^.JIS7KatakanaSend := 0;

  {KanjiIn}
  GetPrivateProfileString(Section,'KanjiIn','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'@')=0 then ts^.KanjiIn := IdKanjiInA
  else ts^.KanjiIn := IdKanjiInB;

  {KanjiOut}
  GetPrivateProfileString(Section,'KanjiOut','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'B')=0 then ts^.KanjiOut := IdKanjiOutB
  else if ts^.AllowWrongSequence and (StrIComp(Temp,'H')=0) then ts^.KanjiOut := IdKanjiOutH
  else ts^.KanjiOut := IdKanjiOutJ;

  {Auto Win Switch VT<->TEK}
  ts^.AutoWinSwitch := GetOnOff(Section,'AutoWinSwitch',FName,false);

  {Terminal ID}
  GetPrivateProfileString(Section,'TerminalID','',
                          Temp,sizeof(Temp),FName);
  ts^.TerminalID := str2id(@TermList,Temp,IdVT100);

  {Title String}
  GetPrivateProfileString(Section,'Title','Tera Term',
                          ts^.Title,SizeOf(ts^.Title),FName);

  {Cursor shape}
  GetPrivateProfileString(Section,'CursorShape','',
                          Temp,SizeOf(Temp),FName);
       if StrIComp(Temp,'vertical'  )=0 then ts^.CursorShape := IdVCur
  else if StrIComp(Temp,'horizontal')=0 then ts^.CursorShape := IdHCur
  else ts^.CursorShape := IdBlkCur;

  {Popup menu}
  ts^.PopupMenu := GetOnOff(Section,'PopupMenu',FName,false);

  {3D dialog box}
  ts^.Dlg3D := GetOnOff(Section,'Ctl3D',FName,false);

  {Enable scroll buffer}
  ts^.EnableScrollBuff :=
    GetOnOff(Section,'EnableScrollBuff',FName,true);

  {Scroll buffer size}
  ts^.SCrollBuffMax :=
    GetPrivateProfileInt(Section,'ScrollBuffSize',100,FName);

  {VT Color}
  GetPrivateProfileString(Section,'VTColor','0,0,0,255,255,255',
                          Temp,SizeOf(Temp),FName);
  for i := 0 to 5 do
    GetNthNum(Temp,i+1,integer(ts^.TmpColor[0,i]));
  for i := 0 to 1 do
    ts^.VTColor[i] := RGB(ts^.TmpColor[0,i*3],
                              ts^.TmpColor[0,i*3+1],
                              ts^.TmpColor[0,i*3+2]);

  {VT Bold Color}
  GetPrivateProfileString(Section,'VTBoldColor','0,0,255,255,255,255',
                          Temp,SizeOf(Temp),FName);
  for i := 0 to 5 do
    GetNthNum(Temp,i+1,integer(ts^.TmpColor[0,i]));
  for i := 0 to 1 do
    ts^.VTBoldColor[i] := RGB(ts^.TmpColor[0,i*3],
                              ts^.TmpColor[0,i*3+1],
                              ts^.TmpColor[0,i*3+2]);

  {VT Blink Color}
  GetPrivateProfileString(Section,'VTBlinkColor','255,0,0,255,255,255',
                          Temp,SizeOf(Temp),FName);
  for i := 0 to 5 do
    GetNthNum(Temp,i+1,integer(ts^.TmpColor[0,i]));
  for i := 0 to 1 do
    ts^.VTBlinkColor[i] := RGB(ts^.TmpColor[0,i*3],
                              ts^.TmpColor[0,i*3+1],
                              ts^.TmpColor[0,i*3+2]);

  {TEK Color}
  GetPrivateProfileString(Section,'TEKColor','0,0,0,255,255,255',
                          Temp,SizeOf(Temp),FName);
  for i := 0 to 5 do
    GetNthNum(Temp,i+1,integer(ts^.TmpColor[0,i]));
  for i := 0 to 1 do
    ts^.TEKColor[i] := RGB(ts^.TmpColor[0,i*3],
                               ts^.TmpColor[0,i*3+1],
                               ts^.TmpColor[0,i*3+2]);

  TmpDC := GetDC(0); {Get screen device context}
  for i := 0 to 1 do
    ts^.VTColor[i] := GetNearestColor(TmpDC, ts^.VTColor[i]);
  for i := 0 to 1 do
    ts^.VTBoldColor[i] := GetNearestColor(TmpDC, ts^.VTBoldColor[i]);
  for i := 0 to 1 do
    ts^.VTBlinkColor[i] := GetNearestColor(TmpDC, ts^.VTBlinkColor[i]);
  for i := 0 to 1 do
    ts^.TEKColor[i] := GetNearestColor(TmpDC, ts^.TEKColor[i]);
  ReleaseDC(0, TmpDC);

  {TEK color emulation}
  ts^.TEKColorEmu :=
    GetOnOff(Section,'TEKColorEmulation',FName,false);

  {VT Font}
  GetPrivateProfileString(Section,'VTFont','FixedSys,6,13,1',
                          Temp,SizeOf(Temp),FName);
  GetNthString(Temp,1,SizeOf(ts^.VTFont),ts^.VTFont);
  GetNthNum(Temp,2,ts^.VTFontSize.x);
  GetNthNum(Temp,3,ts^.VTFontSize.y);
  GetNthNum(Temp,4,ts^.VTFontCharSet);

  {Bold font flag}
  ts^.EnableBold := GetOnOff(Section,'EnableBold',FName,false);

  {TEK Font}
  GetPrivateProfileString(Section,'TEKFont','FixedSys,6,13,1',
                          Temp,SizeOf(Temp),FName);
  GetNthString(Temp,1,SizeOf(ts^.TEKFont),ts^.TEKFont);
  GetNthNum(Temp,2,ts^.TEKFontSize.x);
  GetNthNum(Temp,3,ts^.TEKFontSize.y);
  GetNthNum(Temp,4,ts^.TEKFontCharSet);

  {BS Key}
  GetPrivateProfileString(Section,'BSKey','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'DEL')=0 then ts^.BSKey := IdDEL
                            else ts^.BSKey := IdBS;

  {IME Flag}
  ts^.UseIME := GetOnOff(Section,'IME',FName,true);

  {IME Flag}
  ts^.IMEInline := GetOnOff(Section,'IMEInline',FName,true);

  {Meta Key}
  ts^.MetaKey := GetOnOff(Section,'MetaKey',FName,false);

  {Serial port ID}
  ts^.ComPort :=
    GetPrivateProfileInt(Section,'ComPort',1,FName);
  if (ts^.ComPort<1) or (ts^.ComPort>4) then
    ts^.ComPort := 1;

  {Baud rate}
  GetPrivateProfileString(Section,'BaudRate','',
                          Temp,sizeof(Temp),FName);
  ts^.Baud := str2id(@BaudList,Temp,IdBaud9600);

  {Parity}
  GetPrivateProfileString(Section,'Parity','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'even')=0 then ts^.Parity := IdParityEven
  else if StrIComp(Temp,'odd' )=0 then ts^.Parity := IdParityOdd
  else ts^.Parity := IdParityNone;

  {Data bit}
  GetPrivateProfileString(Section,'DataBit','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'7')=0 then ts^.DataBit := IdDataBit7
  else ts^.DataBit := IdDataBit8;

  {Stop bit}
  GetPrivateProfileString(Section,'StopBit','',
                          Temp,SizeOf(Temp),FName);
  if StrIComp(Temp,'2')=0 then ts^.StopBit := IdStopBit2
  else ts^.StopBit := IdStopBit1;

  {Flow control}
  GetPrivateProfileString(Section,'FlowCtrl','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'x'   )=0 then ts^.Flow := IdFlowX
  else if StrIComp(Temp,'hard')=0 then ts^.Flow := IdFlowHard
  else ts^.Flow := IdFlowNone;

  {Delay per character}
  ts^.DelayPerChar :=
    GetPrivateProfileInt(Section,'DelayPerChar',0,FName);

  {Delay per line}
  ts^.DelayPerLine :=
    GetPrivateProfileInt(Section,'DelayPerLine',0,FName);

  {TCP port num}
  ts^.TCPPort :=
    GetPrivateProfileInt(Section,'TCPPort',23,FName);

  {Telnet flag}
  ts^.Telnet := GetOnOff(Section,'Telnet',FName,true);

  {Telnet terminal type}
  GetPrivateProfileString(Section,'TermType','vt100',ts^.TermType,
                          SizeOf(ts^.TermType),FName);

  {Auto window close flag}
  ts^.AutoWinClose := GetOnOff(Section,'AutoWinClose',FName,true);

  {File transfer binary flag}
  ts^.TransBin := GetOnOff(Section,'TransBinary',FName,false);

  {XMODEM option}
  GetPrivateProfileString(Section,'XmodemOpt','',
                          Temp,SizeOf(Temp),FName);
  if      StrIComp(Temp,'crc'   )=0 then ts^.XmodemOpt := XoptCRC
  else if StrIComp(Temp,'1k')=0 then ts^.XmodemOpt := Xopt1K
  else ts^.XmodemOpt := XoptCheck;

  {XMODEM binary file}
  ts^.XmodemBin := GetOnOff(Section,'XmodemBinary',FName,true);

  {Default directory for file transfer}
  GetPrivateProfileString(Section,'FileDir','',
                          ts^.FileDir,SizeOf(ts^.FileDir),FName);
  StrUpper(ts^.FileDir);
  if StrLen(ts^.FileDir)=0 then
    StrCopy(ts^.FileDir,ts^.HomeDir)
  else begin
    GetCurDir(Temp,0);
    SetCurDir(ts^.FileDir);
    if DosError>0 then
      StrCopy(ts^.FileDir,ts^.HomeDir);
    SetCurDir(Temp);
  end;

  {Debug flag  -- read-only option}
  ts^.debug := GetOnOff(Section,'Debug',FName,false)>0;

  {Kermit log  -- read-only option}
  ts^.KmtLog := GetOnOff(Section,'KmtLog',FName,false)>0;

  {XMODEM log  -- read-only option}
  ts^.XmodemLog := GetOnOff(Section,'XmodemLog',FName,false)>0;

  {ZMODEM log  -- read-only option}
  ts^.ZmodemLog := GetOnOff(Section,'ZmodemLog',FName,false)>0;

  {ZMODEM ESCCTL flag  -- read-only option}
  ts^.ZmodemEscCtl := GetOnOff(Section,'ZmodemEscCtl',FName,false)>0;

  {ZMODEM data subpacket length for sending -- read-only}
  ts^.ZmodemDataLen :=
    GetPrivateProfileInt(Section,'ZmodemDataLen',1024,FName);
  {ZMODEM window size for sending -- read-only}
  ts^.ZmodemWinSize :=
    GetPrivateProfileInt(Section,'ZmodemWinSize',0,FName);

  {Auto ZMODEM activation -- read-only option}
  ts^.ZmodemAuto := GetOnOff(Section,'ZmodemAuto',FName,false)>0;

  {B-Plus log  -- read-only option}
  ts^.BPLog := GetOnOff(Section,'BPLog',FName,false)>0;

  {Quick-VAN log  -- read-only option}
  ts^.QVLog := GetOnOff(Section,'QVLog',FName,false)>0;

  {Quick-VAN window size -- read-only}
  ts^.QVWinSize :=
    GetPrivateProfileInt(Section,'QVWinSize',8,FName);

  {Telnet log  -- read-only option}
  ts^.TelLog := GetOnOff(Section,'TelLog',FName,false)>0;

  {Ctrl code in Kanji -- read-only option}
  ts^.CtrlInKanji := GetOnOff(Section,'CtrlInKanji',FName,true)>0;

  {Scroll threshold -- read-only option}
  ts^.ScrollThreshold :=
    GetPrivateProfileInt(Section,'ScrollThreshold',12,FName);

  {Wrong sequence flag  -- read-only option}
  ts^.AllowWrongSequence :=
    GetOnOff(Section,'AllowWrongSequence',FName,false)>0;

  {8 bit control code flag  -- read-only option}
  ts^.Accept8bitCtrl :=
    GetOnOff(Section,'Accept8bitCtrl',FName,true)>0;

  {TEK GIN Mouse keycode -- read-only option}
  ts^.GINMouseCode :=
    GetPrivateProfileInt(Section,'TEKGINMouseCode',32,FName);

  {Confirm disconnect -- read-only option}
  ts^.ConfirmDisconn :=
    GetOnOff(Section,'ConfirmDisconnect',FName,true);

  {VT Font space --- read-only option}
  GetPrivateProfileString(Section,'VTFontSpace','0,0,0,0',
                          Temp,SizeOf(Temp),FName);
  GetNthNum(Temp,1,ts^.FontDX);
  GetNthNum(Temp,2,ts^.FontDW);
  GetNthNum(Temp,3,ts^.FontDY);
  GetNthNum(Temp,4,ts^.FontDH);
  if ts^.FontDX<0 then
    ts^.FontDX := 0;
  if ts^.FontDW<0 then
    ts^.FontDW := 0;
  ts^.FontDW := ts^.FontDW +
                   ts^.FontDX;
  if ts^.FontDY<0 then
    ts^.FontDY := 0;
  if ts^.FontDH<0 then
    ts^.FontDH := 0;
  ts^.FontDH := ts^.FontDH +
                   ts^.FontDY;

  {Printer Font --- read-only option}
  GetPrivateProfileString(Section,'PrnFont','',
                          Temp,SizeOf(Temp),FName);
  if StrLen(Temp)=0 then
  begin
    ts^.PrnFont[0] := #0;
    ts^.PrnFontSize.X := 0;
    ts^.PrnFontSize.Y := 0;
    ts^.PrnFontCharSet := 0;
  end
  else begin
    GetNthString(Temp,1,SizeOf(ts^.PrnFont),ts^.PrnFont);
    GetNthNum(Temp,2,ts^.PrnFontSize.X);
    GetNthNum(Temp,3,ts^.PrnFontSize.Y);
    GetNthNum(Temp,4,ts^.PrnFontCharSet);
  end;

  {Telnet binary flag -- read-only option}
  ts^.TelBin := GetOnOff(Section,'TelBin',FName,false);

  {Hide title -- read-only option}
  ts^.HideTitle := GetOnOff(Section,'HideTitle',FName,false);

  ts^.Ok := not AskUser;

end;

procedure WriteIniFile(FName: PChar; ts: PTTSet); export;
var
  i: integer;
  NumStr: String[10];
  Temp: array[0..80] of char;
  Temp2: array[0..10] of char;
begin
  {version}
  WritePrivateProfileString(Section,'Version','1.3',FName);

  {Language}
  if ts^.Language=IdJapanese then
    StrCopy(Temp,'Japanese')
  else
    StrCopy(Temp,'English');
  WritePrivateProfileString(Section,'Language',Temp,FName);

  {Port type}
  case ts^.PortType of
    IdSerial: StrCopy(Temp,'serial')
  else {IdFile -> IdTCPIP}
    StrCopy(Temp,'tcpip');
  end;
  WritePrivateProfileString(Section,'Port',Temp,FName);

  {VT win position}
  Str(ts^.VTPos.x,NumStr);
  StrPCopy(Temp,NumStr);
  StrCat(Temp,',');

  Str(ts^.VTPos.y,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);

  WritePrivateProfileString(Section,'VTPos',Temp,FName);

  {TEK win position}
  Str(ts^.TEKPos.x,NumStr);
  StrPCopy(Temp,NumStr);
  StrCat(Temp,',');

  Str(ts^.TEKPos.y,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);

  WritePrivateProfileString(Section,'TEKPos',Temp,FName);

  {VT terminal size }

  Str(ts^.TerminalWidth,NumStr);
  StrPCopy(Temp,NumStr);
  StrCat(Temp,',');

  Str(ts^.TerminalHeight,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);
  WritePrivateProfileString(Section,'TerminalSize',Temp,FName);

  {Terminal size = Window size}
  WriteOnOff(Section,'TermIsWin',FName,ts^.TermIsWin);

  {Auto window resize flag}
  WriteOnOff(Section,'AutoWinResize',FName,ts^.AutoWinResize);

  {CR Receive}
  if ts^.CRReceive = IdCRLF then StrCopy(Temp,'CRLF')
                                 else StrCopy(Temp,'CR');
  WritePrivateProfileString(Section,'CRReceive',Temp,FName);

  {CR Send}
  if ts^.CRSend = IdCRLF then StrCopy(Temp,'CRLF')
                              else StrCopy(Temp,'CR');
  WritePrivateProfileString(Section,'CRSend',Temp,FName);

  {Local echo}
  WriteOnOff(Section,'LocalEcho',FName,ts^.LocalEcho);

  {Answerback}
  if not ts^.BPAuto then
  begin
    Str2Hex(ts^.Answerback,Temp,ts^.AnswerbackLen,
            SizeOf(Temp)-1,true);
    WritePrivateProfileString(Section,'Answerback',Temp,FName);
  end;

  {Kanji Code (receive) }
  case ts^.KanjiCode of
    IdEUC: StrCopy(Temp,'EUC');
    IdJIS: StrCopy(Temp,'JIS');
  else
    StrCopy(Temp,'SJIS');
  end;
  WritePrivateProfileString(Section,'KanjiReceive',Temp,FName);

  {Katakana (receive) }
  case ts^.JIS7Katakana of
    1: StrCopy(Temp,'7');
  else
    StrCopy(Temp,'8');
  end;
  WritePrivateProfileString(Section,'KatakanaReceive',Temp,FName);

  {Kanji Code (transmit) }
  case ts^.KanjiCodeSend of
    IdEUC: StrCopy(Temp,'EUC');
    IdJIS: StrCopy(Temp,'JIS');
  else
    StrCopy(Temp,'SJIS');
  end;
  WritePrivateProfileString(Section,'KanjiSend',Temp,FName);

  {Katakana (transmit) }
  case ts^.JIS7KatakanaSend of
    1: StrCopy(Temp,'7');
  else
    StrCopy(Temp,'8');
  end;
  WritePrivateProfileString(Section,'KatakanaSend',Temp,FName);

  {KanjiIn}
  case ts^.KanjiIn of
    IdKanjiInA: StrCopy(Temp,'@');
  else
    StrCopy(Temp,'B');
  end;
  WritePrivateProfileString(Section,'KanjiIn',Temp,FName);

  {KanjiOut}
  case ts^.KanjiOut of
    IdKanjiOutB: StrCopy(Temp,'B');
    IdKanjiOutH: StrCopy(Temp,'H');
  else
    StrCopy(Temp,'J');
  end;
  WritePrivateProfileString(Section,'KanjiOut',Temp,FName);

  {AutoWinChange VT<->TEK}
  WriteOnOff(Section,'AutoWinSwitch',FName,ts^.AutoWinSwitch);

  {Terminal ID}
  id2str(@TermList,ts^.TerminalID,IdVT100,Temp);
  WritePrivateProfileString(Section,'TerminalID',Temp,FName);

  {Title text}
  WritePrivateProfileString(Section,'Title',ts^.Title,FName);

  {Cursor shape}
  case ts^.CursorShape of
    IdVCur  : StrCopy(Temp,'vertical');
    IdHCur  : StrCopy(Temp,'horizontal');
  else
    StrCopy(Temp,'block');
  end;
  WritePrivateProfileString(Section,'CursorShape',Temp,FName);

  {Popup menu}
  WriteOnOff(Section,'PopupMenu',FName,ts^.PopupMenu);

  {3D dialog box}
  WriteOnOff(Section,'Ctl3D',FName,ts^.Dlg3D);

  {Enable scroll buffer}
  WriteOnOff(Section,'EnableScrollBuff',FName,ts^.EnableScrollBuff);

  {Scroll buffer size}
  Str(ts^.ScrollBuffMax,NumStr);
  StrPCopy(Temp,NumStr);
  WritePrivateProfileString(Section,'ScrollBuffSize',Temp,FName);

  {VT Color}
  for i := 0 to 1 do
  begin
    ts^.TmpColor[0,i*3]   := GetRValue(ts^.VTColor[i]);
    ts^.TmpColor[0,i*3+1] := GetGValue(ts^.VTColor[i]);
    ts^.TmpColor[0,i*3+2] := GetBValue(ts^.VTColor[i]);
  end;
  Temp[0] := #0;
  for i := 0 to 5 do
  begin
    Str(ts^.TmpColor[0,i],NumStr);
    StrPCopy(Temp2,NumStr);
    StrCat(Temp,Temp2);
    if i<>5 then StrCat(Temp,',');
  end;
  WritePrivateProfileString(Section,'VTColor',Temp,FName);

  {VT bold color}
  for i := 0 to 1 do
  begin
    ts^.TmpColor[0,i*3]   := GetRValue(ts^.VTBoldColor[i]);
    ts^.TmpColor[0,i*3+1] := GetGValue(ts^.VTBoldColor[i]);
    ts^.TmpColor[0,i*3+2] := GetBValue(ts^.VTBoldColor[i]);
  end;
  Temp[0] := #0;
  for i := 0 to 5 do
  begin
    Str(ts^.TmpColor[0,i],NumStr);
    StrPCopy(Temp2,NumStr);
    StrCat(Temp,Temp2);
    if i<>5 then StrCat(Temp,',');
  end;
  WritePrivateProfileString(Section,'VTBoldColor',Temp,FName);

  {VT blink color}
  for i := 0 to 1 do
  begin
    ts^.TmpColor[0,i*3]   := GetRValue(ts^.VTBlinkColor[i]);
    ts^.TmpColor[0,i*3+1] := GetGValue(ts^.VTBlinkColor[i]);
    ts^.TmpColor[0,i*3+2] := GetBValue(ts^.VTBlinkColor[i]);
  end;
  Temp[0] := #0;
  for i := 0 to 5 do
  begin
    Str(ts^.TmpColor[0,i],NumStr);
    StrPCopy(Temp2,NumStr);
    StrCat(Temp,Temp2);
    if i<>5 then StrCat(Temp,',');
  end;
  WritePrivateProfileString(Section,'VTBlinkColor',Temp,FName);

  {TEK Color}
  for i := 0 to 1 do
  begin
    ts^.TmpColor[0,i*3]   := GetRValue(ts^.TEKColor[i]);
    ts^.TmpColor[0,i*3+1] := GetGValue(ts^.TEKColor[i]);
    ts^.TmpColor[0,i*3+2] := GetBValue(ts^.TEKColor[i]);
  end;
  Temp[0] := #0;
  for i := 0 to 5 do
  begin
    Str(ts^.TmpColor[0,i],NumStr);
    StrPCopy(Temp2,NumStr);
    StrCat(Temp,Temp2);
    if i<>5 then StrCat(Temp,',');
  end;
  WritePrivateProfileString(Section,'TEKColor',Temp,FName);

  {TEK color emulation}
  if ts^.TEKColorEmu = 1 then StrCopy(Temp,'on')
                             else StrCopy(Temp,'off');
  WritePrivateProfileString(Section,'TEKColorEmulation',Temp,FName);

  {VT Font}
  StrCopy(Temp,ts^.VTFont);
  StrCat(Temp,',');

  Str(ts^.VTFontSize.x,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);
  StrCat(Temp,',');

  Str(ts^.VTFontSize.y,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);
  StrCat(Temp,',');

  Str(ts^.VTFontCharSet,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);

  WritePrivateProfileString(Section,'VTFont',Temp,FName);

  {Enable bold font flag}
  WriteOnOff(Section,'EnableBold',FName,ts^.EnableBold);

  {TEK Font}
  StrCopy(Temp,ts^.TEKFont);
  StrCat(Temp,',');

  Str(ts^.TEKFontSize.x,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);
  StrCat(Temp,',');

  Str(ts^.TEKFontSize.y,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);
  StrCat(Temp,',');

  Str(ts^.TEKFontCharSet,NumStr);
  StrPCopy(Temp2,NumStr);
  StrCat(Temp,Temp2);

  WritePrivateProfileString(Section,'TEKFont',Temp,FName);

  {BS Key}
  if ts^.BSKey = IdDEL then StrCopy(Temp,'DEL')
                            else StrCopy(Temp,'BS');
  WritePrivateProfileString(Section,'BSKey',Temp,FName);

  {IME Flag}
  WriteOnOff(Section,'IME',FName,ts^.UseIME);

  {IME Flag}
  WriteOnOff(Section,'IMEInline',FName,ts^.IMEInline);

  {Meta key}
  WriteOnOff(Section,'MetaKey',FName,ts^.MetaKey);

  {Serial port ID}
  case ts^.ComPort of
    IdCOM2: StrCopy(Temp,'2');
    IdCOM3: StrCopy(Temp,'3');
    IdCOM4: StrCopy(Temp,'4');
  else
    StrCopy(Temp,'1');
  end;
  WritePrivateProfileString(Section,'ComPort',Temp,FName);

  {Baud rate}
  id2str(@BaudList,ts^.Baud,IdBaud9600,Temp);
  WritePrivateProfileString(Section,'BaudRate',Temp,FName);

  {Parity}
  case ts^.Parity of
    IdParityEven: StrCopy(Temp,'even');
    IdParityOdd:  StrCopy(Temp,'odd');
  else
    StrCopy(Temp,'none');
  end;
  WritePrivateProfileString(Section,'Parity',Temp,FName);

  {Data bit}
  case ts^.DataBit of
    IdDataBit7: StrCopy(Temp,'7');
  else
    StrCopy(Temp,'8');
  end;
  WritePrivateProfileString(Section,'DataBit',Temp,FName);

  {Stop bit}
  case ts^.StopBit of
    IdStopBit2: StrCopy(Temp,'2');
  else
    StrCopy(Temp,'1');
  end;
  WritePrivateProfileString(Section,'StopBit',Temp,FName);

  {Flow control}
  case ts^.Flow of
    IdFlowX:    StrCopy(Temp,'x');
    IdFlowHard: StrCopy(Temp,'hard');
  else
    StrCopy(Temp,'none');
  end;
  WritePrivateProfileString(Section,'FlowCtrl',Temp,FName);

  {Delay per character}
  Str(ts^.DelayPerChar,NumStr);
  StrPCopy(Temp,NumStr);
  WritePrivateProfileString(Section,'DelayPerChar',Temp,FName);

  {Delay per line}
  Str(ts^.DelayPerLine,NumStr);
  StrPCopy(Temp,NumStr);
  WritePrivateProfileString(Section,'DelayPerLine',Temp,FName);

  {TCP port num}
  Str(ts^.TCPPort,NumStr);
  StrPCopy(Temp,NumStr);
  WritePrivateProfileString(Section,'TCPPort',Temp,FName);

  {Telnet flag}
  WriteOnOff(Section,'Telnet',FName,ts^.Telnet);

  {Telnet terminal type}
  WritePrivateProfileString(Section,'TermType',ts^.TermType,FName);

  {Auto close flag}
  WriteOnOff(Section,'AutoWinClose',FName,ts^.AutoWinClose);

  {File transfer binary flag}
  WriteOnOff(Section,'TransBin',FName,ts^.TransBin);

  {XMODEM option}
  case ts^.XmodemOpt of
    XoptCRC: StrCopy(Temp,'crc');
    Xopt1K:  StrCopy(Temp,'1k');
  else
    StrCopy(Temp,'checksum');
  end;
  WritePrivateProfileString(Section,'XmodemOpt',Temp,FName);

  {XMODEM binary flag}
  WriteOnOff(Section,'XmodemBin',FName,ts^.XmodemBin);

  {Default directory for file transfer}
  WritePrivateProfileString(Section,'FileDir',ts^.FileDir,FName);
end;

const
  VTEditor='VT editor keypad';
  VTNumeric='VT numeric keypad';
  VTFunction='VT function keys';
  ShortCut='Shortcut keys';

function GetInt(Sect, Key: PChar; Def: word; FName: PChar): word;
var
  Temp: array[0..10] of char;
  c: integer;
  Num: word;
begin
  GetPrivateProfileString(Sect,Key,'',
                          Temp,SizeOf(Temp),FName);
  if Temp[0]=#0 then
    GetInt := Def
  else if StrIComp(Temp,'off')=0 then
    GetInt := $FFFF
  else begin
    Val(StrPas(@Temp[0]),Num,c);
    if c <> 0 then Num := $FFFF;
    GetInt := Num;
  end;
end;

procedure ReadKeyboardCnf(FName: PChar; KeyMap: PKeyMap); export;
var
  i, j, Ptr: integer;
  EntName: array[0..6] of char;
  TempStr: array[0..100] of char;
  KStr: PChar;
begin
with KeyMap^ do begin
  {Key Codes for VT editor keypad}
  Map[IdUp-1] := GetInt(VTEditor,'Up',328,FName);

  Map[IdDown-1] := GetInt(VTEditor,'Down',336,FName);

  Map[IdRight-1] := GetInt(VTEditor,'Right',333,FName);

  Map[IdLeft-1] := GetInt(VTEditor,'Left',331,FName);

  Map[IdFind-1] := GetInt(VTEditor,'Find',335,FName);

  Map[IdInsert-1] := GetInt(VTEditor,'Insert',338,FName);

  Map[IdRemove-1] := GetInt(VTEditor,'Remove',339,FName);

  Map[IdSelect-1] := GetInt(VTEditor,'Select',327,FName);

  Map[IdPrev-1] := GetInt(VTEditor,'Prev',329,FName);

  Map[IdNext-1] := GetInt(VTEditor,'Next',337,FName);

  {Key Codes for VT numeric keypad}
  Map[Id0-1] := GetInt(VTNumeric,'Num0',82,FName);

  Map[Id1-1] := GetInt(VTNumeric,'Num1',79,FName);

  Map[Id2-1] := GetInt(VTNumeric,'Num2',80,FName);

  Map[Id3-1] := GetInt(VTNumeric,'Num3',81,FName);

  Map[Id4-1] := GetInt(VTNumeric,'Num4',75,FName);

  Map[Id5-1] := GetInt(VTNumeric,'Num5',76,FName);

  Map[Id6-1] := GetInt(VTNumeric,'Num6',77,FName);

  Map[Id7-1] := GetInt(VTNumeric,'Num7',71,FName);

  Map[Id8-1] := GetInt(VTNumeric,'Num8',72,FName);

  Map[Id9-1] := GetInt(VTNumeric,'Num9',73,FName);

  Map[IdMinus-1] := GetInt(VTNumeric,'NumMinus',1102,FName);

  Map[IdComma-1] := GetInt(VTNumeric,'NumComma',78,FName);

  Map[IdPeriod-1] := GetInt(VTNumeric,'NumPeriod',83,FName);

  Map[IdEnter-1] := GetInt(VTNumeric,'NumEnter',284,FName);

  Map[IdPF1-1] := GetInt(VTNumeric,'PF1',325,FName);

  Map[IdPF2-1] := GetInt(VTNumeric,'PF2',309,FName);

  Map[IdPF3-1] := GetInt(VTNumeric,'PF3',55,FName);

  Map[IdPF4-1] := GetInt(VTNumeric,'PF4',74,FName);

  {Key Codes for VT function keys}
  Map[IdF6-1] := GetInt(VTFunction,'F6',64,FName);

  Map[IdF7-1] := GetInt(VTFunction,'F7',65,FName);

  Map[IdF8-1] := GetInt(VTFunction,'F8',66,FName);

  Map[IdF9-1] := GetInt(VTFunction,'F9',67,FName);

  Map[IdF10-1] := GetInt(VTFunction,'F10',68,FName);

  Map[IdF11-1] := GetInt(VTFunction,'F11',87,FName);

  Map[IdF12-1] := GetInt(VTFunction,'F12',88,FName);

  Map[IdF13-1] := GetInt(VTFunction,'F13',573,FName);

  Map[IdF14-1] := GetInt(VTFunction,'F14',574,FName);

  Map[IdHelp-1] := GetInt(VTFunction,'Help',575,FName);

  Map[IdDo-1] := GetInt(VTFunction,'Do',576,FName);

  Map[IdF17-1] := GetInt(VTFunction,'F17',577,FName);

  Map[IdF18-1] := GetInt(VTFunction,'F18',578,FName);

  Map[IdF19-1] := GetInt(VTFunction,'F19',579,FName);

  Map[IdF20-1] := GetInt(VTFunction,'F20',580,FName);

  Map[IdAccEditCopy-1] := GetInt(ShortCut,'EditCopy',65535,FName);

  Map[IdAccEditPaste-1] := GetInt(ShortCut,'EditPaste',65535,FName);

  Map[IdAccEditPasteCR-1] := GetInt(ShortCut,'EditPasteCR',65535,FName);

  Map[IdAccLineUp-1] := GetInt(ShortCut,'LineUp',1352,FName);

  Map[IdAccLineDown-1] := GetInt(ShortCut,'LineDown',1360,FName);

  Map[IdAccPageUp-1] := GetInt(ShortCut,'PageUp',1353,FName);

  Map[IdAccPageDown-1] := GetInt(ShortCut,'PageDown',1361,FName);

  Map[IdAccBuffTop-1] := GetInt(ShortCut,'BuffTop',1351,FName);

  Map[IdAccBuffBottom-1] := GetInt(ShortCut,'BuffBottom',1359,FName);

  Map[IdAccSendBreak-1] := GetInt(ShortCut,'SendBreak',65535,FName);

  {user keys}

  for i := 0 to NumOfUserKey-1 do
  begin
    Map[IdUser1+i-1] := $FFFF;
    UserKeyPtr[i] := 0;
    UserKeyLen[i] := 0;
  end;

  Ptr := 0;

  StrCopy(EntName,'User');
  i := IdUser1;
  repeat
    if i < IdUser1+9 then
    begin
      EntName[4] := char(i-IdUser1+$31);
      EntName[5] := #0;
    end
    else begin
      EntName[4] := char((i-IdUser1+1) div 10 + $30);
      EntName[5] := char((i-IdUser1+1) mod 10 + $30);
      EntName[6] := #0;
    end;

    GetPrivateProfileString('User keys',EntName,'',
                             TempStr,SizeOf(TempStr),FName);
    if StrLen(TempStr) > 0 then
    begin
      {scan code}
      GetNthNum(TempStr,1,integer(Map[i-1]));
      {conversion flag}
      GetNthNum(TempStr,2,j);
      if j=0 then UserKeyConv[i-IdUser1] := 0
             else UserKeyConv[i-IdUser1] := 1;
      {key string}
      UserKeyPtr[i-IdUser1] := Ptr;
      KStr := StrScan(TempStr,',');
      if KStr<>nil then
        KStr := StrScan(@KStr[1],',');
      if KStr<>nil then
      begin
        UserKeyLen[i-IdUser1] :=
          Hex2Str(@KStr[1],@UserKeyStr[Ptr],KeyStrMax-Ptr+1);
        Ptr := Ptr + UserKeyLen[i-IdUser1];
      end;
    end;

    inc(i)
  until (i > IdKeyMax) or (StrLen(TempStr)=0) or (Ptr>KeyStrMax);

end;
end;

{copy hostlist from source IniFile to dest IniFile}
procedure CopyHostList(IniSrc, IniDest: PChar); export;
var
  i: integer;
  EntName: array[0..6] of char;
  TempHost: array[0..HostNameMaxLength] of char;
begin
  if StrIComp(IniSrc,IniDest)=0 then exit;

  WritePrivateProfileString('Hosts',nil,nil,IniDest);
  StrCopy(EntName,'Host');

  i := 1;
  repeat
    if i < 10 then
    begin
      EntName[4] := char(i+$30);
      EntName[5] := #0;
    end
    else begin
      EntName[4] := char(i div 10 + $30);
      EntName[5] := char(i mod 10 + $30);
      EntName[6] := #0;
    end;

    {Get one hostname from file IniSrc}
    GetPrivateProfileString('Hosts',EntName,'',
                            TempHost,SizeOf(TempHost),IniSrc);
    {Copy it to file IniDest}
    if StrLen(TempHost) > 0 then
      WritePrivateProfileString('Hosts',EntName,TempHost,IniDest);
    inc(i)
  until (i > 99) or (StrLen(TempHost)=0);

end;

procedure ParseParam(Param: PChar; ts: PTTSet; DDETopic: PChar); export;

  function NextParam(var i: integer; Temp: PChar; Size: integer): boolean;
  var
    j: integer;
    c: char;
    Quoted: BOOL;
  begin
    NextParam := false;
    if i >= StrLen(Param) then exit;
    j := 0;

    while Param[i]=' ' do
      inc(i);

    Quoted := FALSE;
    c := Param[i];
    inc(i);
    while (c<>#0) and (Quoted or (c<>' ')) and
          (Quoted or (c<>';')) and (j<Size-1) do
    begin
      if c='"' then
        Quoted := not Quoted;
      Temp[j] := c;
      inc(j);
      c := Param[i];
      inc(i);
    end;
    if not Quoted and (c=';') then
      dec(i);

    Temp[j] := #0;
    NextParam := strlen(Temp)>0;
  end;

  procedure DeQuote(Source, Dest: PCHAR);
  var
    i, j: integer;
    q, c: char;
  begin
    Dest[0] := #0;
    if Source[0] = #0 then exit;
    i := 0;
    {quoting char}
    q := Source[i];
    {only " and ' are used as quoting char}
    if ((q<>#$22) and (q<>#$27)) then
      q := #0
    else
      inc(i);

    c := Source[i];
    inc(i);
    j := 0;
    while (c<>#0) and (c<>q) do
    begin
      Dest[j] := c;
      inc(j);
      c := Source[i];
      inc(i);
    end;

    Dest[j] := #0;
  end;

  procedure ConvFName(Temp, DefExt, FName: PChar);
  var
   DirLen, FNPos: integer;
  begin
    FName[0] := #0;
    if not GetFileNamePosJ(Temp,DirLen,FNPos) then exit;
    FitFileName(@Temp[FNPos],DefExt);
    if DirLen=0 then
    begin
      StrCopy(FName,ts^.HomeDir);
      StrCat(FName,'\');
    end;
    StrCat(FName,Temp);
    StrUpperJ(FName);
  end;

var
 i, pos, c: integer;
 b: char;
 Temp: array[0..fsPathName+3] of char;
 Temp2: array[0..fsPathName] of char;
 PasStr: string[10];
 TempDir: array[0..fsPathName] of char;
 ParamPort: word;
 ParamCom: word;
 ParamTCP: word;
 ParamTel: word;
 ParamBin: word;
begin
  ts^.HostName[0] := #0;
  ts^.KeyCnfFN[0] := #0;
  ParamPort := 0;
  ParamTel := 2;
  ParamBin := 2; 
  ParamTCP := 65535;
  ParamCOM := 0;

  {Get command line parameters}
  if DDETopic <> nil then
    DDETopic[0] := #0;
  i := 0;
  while NextParam(i, Temp, SizeOf(Temp)) do
  begin
    if StrLIComp(Temp,'/B',2)=0 then {telnet binary}
    begin
      ParamPort := IdTCPIP;
      ParamBin := 1;
    end
    else if StrLIComp(Temp,'/C=',3)=0 then {COM port num}
    begin
      ParamPort := IdSerial;
      ParamCom := byte(Temp[3])-$30;
      if (ParamCom<1) or (ParamCom>4) then
        ParamCom := 0;
    end
    else if StrLIComp(Temp,'/D=',3)=0 then
    begin
      if DDETopic <> nil then
        strcopy(DDETopic,@Temp[3]);
    end
    else if StrLIComp(Temp,'/F=',3)=0 then {setup filename}
    begin
      Dequote(@Temp[3],Temp2);
      if strlen(Temp2) > 0 then
      begin
        ConvFName(Temp2,'.INI',Temp);
        if stricomp(ts^.SetupFName,Temp)<>0 then
        begin
          strcopy(ts^.SetupFName,Temp);
          ReadIniFile(ts^.SetupFName,ts);
        end;
      end;
    end
    else if StrLIComp(Temp,'/FD=',4)=0 then {file transfer directory}
    begin
      Dequote(@Temp[4],Temp2);
      if strlen(Temp2)>0 then
      begin
        GetCurDir(TempDir,0);
        SetCurDir(Temp2);
        if DosError=0 then
          StrCopy(ts^.FileDir,Temp2);
        SetCurDir(TempDir);
        StrUpperJ(ts^.FileDir);
      end;
    end
    else if StrLIComp(Temp,'/H',2)=0 then {hide title bar}
    begin
      ts^.HideTitle := 1;
    end
    else if StrLIComp(Temp,'/K=',3)=0 then {Keyboard setup file}
    begin
      Dequote(@Temp[3],Temp2);
      ConvFName(Temp2, '.CNF', ts^.KeyCnfFN)
    end
    else if (StrLIComp(Temp,'/KR=',4)=0) or
            (StrLIComp(Temp,'/KT=',4)=0) then {kanji code}
    begin
      if StrLIComp(@Temp[4],'SJIS',4)=0 then
        c := IdSJIS
      else if StrLIComp(@Temp[4],'EUC',3)=0 then
        c := IdEUC
      else if StrLIComp(@Temp[4],'JIS',3)=0 then
        c := IdJIS
      else
        c := -1;
      if c<>-1 then
      begin
        if StrLIComp(Temp,'/KR=',4)=0 then
          ts^.KanjiCode := c
        else
          ts^.KanjiCodeSend := c;
      end;
    end
    else if StrLIComp(Temp,'/P=',3)=0 then {TCP port num}
    begin
      ParamPort := IdTCPIP;
      Val(StrPas(@Temp[3]),ParamTCP,c);
      if c <> 0 then ParamTCP := 65535;
    end
    else if StrLIComp(Temp,'/R=',3)=0 then {Replay filename}
    begin
      Dequote(@Temp[3],Temp2);
      ConvFName(Temp2, '', ts^.HostName);
      if StrLen(ts^.HostName)>0 then
        ParamPort := IdFile;
    end
    else if StrLIComp(Temp,'/T=0',4)=0 then {telnet disable}
    begin
      ParamPort := IdTCPIP;
      ParamTel := 0;
    end
    else if StrLIComp(Temp,'/T=1',4)=0 then {telnet enable}
    begin
      ParamPort := IdTCPIP;
      ParamTel := 1;
    end
    else if StrLIComp(Temp,'/W=',3)=0 then {Window title}
    begin
      Dequote(@Temp[3],ts^.Title);
    end
    else if StrLIComp(Temp,'/X=',3)=0 then {Window pos (X)}
    begin
      Val(StrPas(@Temp[3]),pos,c);
      if c=0 then
      begin
        ts^.VTPos.x := pos;
        if ts^.VTPos.y=CW_USEDEFAULT then
          ts^.VTPos.y := 0;
      end;
    end
    else if StrLIComp(Temp,'/Y=',3)=0 then {Window pos (Y)}
    begin
      Val(StrPas(@Temp[3]),pos,c);
      if c=0 then
      begin
        ts^.VTPos.y := pos;
        if ts^.VTPos.x=CW_USEDEFAULT then
          ts^.VTPos.x := 0;
      end;
    end
    else if (Temp[0]<>'/') and (StrLen(Temp)>0) then
    begin
      ParamPort := IdTCPIP;
      StrCopy(ts^.HostName,Temp); {host name}
    end;
  end;

  if (ts^.HostName[0] <> #0) and
     (ParamPort=IdTCPIP) then
  begin
    i := 0;
    repeat
      b := ts^.HostName[i];
      inc(i);
    until (b=#0) or (b=':');
    if b=':' then
    begin
      ts^.HostName[i-1] := #0;
      Val(StrPas(@ts^.HostName[i]),ParamTCP,c);
      if c <> 0 then ParamTCP := 65535;
    end;
  end;

  case ParamPort of
    IdTCPIP: begin
      ts^.PortType := IdTCPIP;
      if ParamTel<2 then ts^.Telnet := ParamTel;
      if ParamBin<2 then ts^.TelBin := ParamBin;
      if ParamTCP<65535 then ts^.TCPPort := ParamTCP;
    end;
    IdSerial: begin
      ts^.PortType := IdSerial;
      if (ParamCOM>0) and (ParamCom<5) then
        ts^.ComPort := ParamCom;
    end;
    IdFile:
      ts^.PortType := IdFile;
  end;
end;

exports

  ReadIniFile     index 1,
  WriteIniFile    index 2,
  ReadKeyboardCnf index 3,
  CopyHostList    index 4,
  ParseParam      index 5;

begin
end.