{Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved.}

{TERATERM.EXE, VT escape sequences}

procedure VTWindow.EscapeSequence(b: byte);
begin
  case b of
    NUL..US: ParseControl(b);
    $20..$2F: begin
                if Icount<IntCharMax then inc(Icount);
                IntChar[Icount] := b;
              end;
    $30..$7E: ParseEscape(b);
    $80..$9F: ParseControl(b);
  end;
  JustAfterESC := false;
end;

procedure VTWindow.ControlSequence(b: byte);
begin
  case b of
    NUL..US: ParseControl(b);
    $20..$2F: begin
                if Icount<IntCharMax then inc(Icount);
                IntChar[Icount] := b;
              end;
    $30..$39: begin
                if Param[NParam] < 0 then Param[NParam] := 0; 
                if Param[NParam]<1000 then
                  Param[NParam] := Param[NParam]*10 + b - $30;
              end;
    $3B: if NParam < NParamMax then
         begin
           Inc(NParam);
           Param[NParam] := -1;
         end;
    $3C..$3F: if FirstPrm then Prv := b;
    $40..$7E: ParseCS(b);
    $80..$9F: ParseControl(b);
  end;
  FirstPrm := false;
end;

procedure VTWindow.ParseControl(b: byte); {b is control char}
begin
  if (b>=$80) then {C1 char}
  begin
    {English mode}    
    if (ts.Language=IdEnglish) then
    begin
      if ts.TerminalId<IdVT220J then
      begin
        DispChar(b); {Disp C1 char in VT100 mode}
        exit;
      end;
    end
    else begin {Japanese mode}    
      if not ts.Accept8bitCtrl then
        exit; {ignore C1 char}
      {C1 chars are interpreted as C0 chars in VT100 mode}
      if ts.TerminalID<IdVT220J then
        b := b and $7F;
    end;
  end;

  case b of
    {C0 group}
    ENQ: CommBinaryOut(@cv,ts.AnswerBack,ts.AnswerBackLen);
    BEL: messagebeep(0);
    BS: BackSpace;
    HT: Tab;
    LF..VT: LineFeed(b);
    FF: if (ts.AutoWinSwitch>0) and JustAfterESC then
        begin
          CommInsert1Byte(@cv,b);
          CommInsert1Byte(@cv,ESC);
          ChangeEmu := IdTEK;  {Enter TEK Mode}
        end
        else
          LineFeed(b);
    CR: begin
          CarriageReturn;
          if ts.CRReceive=IdCRLF then
             CommInsert1Byte(@cv,LF);
        end;
    SO: begin
          if (ts.Language=IdJapanese) and
             (ts.KanjiCode=IdJIS) and
             (ts.JIS7Katakana=1) then Gn[1] := IdKatakana;
          Glr[0] := 1; {LS1}
        end;
    SI: Glr[0] := 0; {LS0}
    DLE: if ts.BPAuto then
      ParseMode := ModeDLE; {Auto B-Plus activation}
    CAN: if ts.ZmodemAuto then
        ParseMode := ModeCAN {Auto ZMODEM activation}
      else
        ParseMode := ModeFirst;
    SUB: ParseMode := ModeFirst;
    ESC: begin
           ICount := 0;
           JustAfterESC := true;
           ParseMode := ModeESC;
         end;
    FS..US: if ts.AutoWinSwitch>0 then
        begin
          CommInsert1Byte(@cv,b);
          ChangeEmu := IdTEK;  {Enter TEK Mode}
        end;

    {C1 group}
    IND: LineFeed(0);
    NEL: begin
           LineFeed(0);
           CarriageReturn;
         end;
    HTS: SetTabStop;
    RI: CursorUpWithScroll;
    SS2: begin
           GLtmp := 2;
           SSflag := true;
         end;
    SS3: begin
           GLtmp := 3;
           SSflag := true;
         end;
    DCS: begin
           SavedMode := ParseMode;
           ESCFlag := false;
           NParam := 1;
           Param[1] := -1;
           ParseMode := ModeDCS;
         end;
    SOS: begin
           SavedMode := ParseMode;
           ESCFlag := false;
           ParseMode := ModeSOS;
         end;
    CSI: begin
           ICount := 0;
           FirstPrm := true;
           NParam := 1;
           Param[1] := -1;
           Prv := 0;
           ParseMode := ModeCSI;
         end;
    OSC..APC: begin
           SavedMode := ParseMode;
           ESCFlag := false;
           ParseMode := ModeSOS;
         end;

  end;
end;

procedure VTWindow.DeviceControl(b: byte);
begin
  if (ESCFlag and (b=Ord('\'))) or (b=ST) then
  begin
    ESCFlag := false;
    ParseMode := SavedMode;
    exit;
  end;

  if b=ESC then
  begin
    ESCFlag := true;
    exit;
  end
  else ESCFlag := false;

  case b of
    NUL..US: ParseControl(b);
    $30..$39: begin
                if Param[NParam] < 0 then Param[NParam] := 0; 
                if Param[NParam]<1000 then
                  Param[NParam] := Param[NParam]*10 + b - $30;
              end;
    $3B: if NParam < NParamMax then
         begin
           Inc(NParam);
           Param[NParam] := -1;
         end;
    $40..$7E: begin
                if b=ord('|') then
                begin
                  ParseMode := ModeDCUserKey;
                  if Param[1] < 1 then ClearUserKey;
                  WaitKeyId := true;
                  NewKeyId := 0;
                end
                else ParseMode := ModeSOS;
              end;
  end;

end;

procedure VTWindow.DCUserKey(b: byte);

  function ConvHexChar(b: byte): byte;
  begin
    case b of
      $31..$39: ConvHexChar := b - $30;
      $41..$46: ConvHexChar := b - $37;
      $61..$66: ConvHexChar := b - $57;
    else
      ConvHexChar := 0;
    end;
  end;

begin
  if (ESCFlag and (b=Ord('\'))) or (b=ST) then
  begin
    if not WaitKeyId then DefineUserKey;
    ESCFlag := false;
    ParseMode := SavedMode;
    exit;
  end;

  if b=ESC then
  begin
    ESCFlag := true;
    exit;
  end
  else ESCFlag := false;

  if WaitKeyId then
    case b of
      $30..$39: if NewKeyId<1000 then
                    NewKeyId := NewKeyId*10 + b - $30;
      $2F: begin
             WaitKeyId := false;
             WaitHi := true;
             NewKeyLen := 0;
           end;
    end
  else
    if b=$3B then
    begin
      DefineUserKey;
      WaitKeyId := true;
      NewKeyId := 0;
    end
    else
      if NewKeyLen < FuncKeyStrMax then
        if WaitHi then
        begin
          NewKeyStr[NewKeyLen] := ConvHexChar(b) shl 4;
          WaitHi := false;
        end
        else begin
          NewKeyStr[NewKeyLen] := NewKeyStr[NewKeyLen] +
                                    ConvHexChar(b);
          WaitHi := true;
          inc(NewKeyLen);
        end;

end;

procedure VTWindow.IgnoreString(b: byte);
begin

  if (ESCFlag and (b=Ord('\'))) or (b=ST) then
    ParseMode := SavedMode;

  if b=ESC then ESCFlag := true
           else ESCFlag := false;

end;

procedure VTWindow.SaveCursor;
begin
  SavedCursor.X := CursorX;
  SavedCursor.Y := CursorY;
  SavedAttr := CharAttr;
  SavedAttr2 := CharAttr2;
end;

procedure VTWindow.RestoreCursor;
begin
  UpdateStr;
  if SavedCursor.X > NumOfColumns-1 then
     SavedCursor.X := NumOfColumns-1;
  if SavedCursor.Y > NumOfLines-1 then
     SavedCursor.Y := NumOfLines-1;
  MoveCursor(SavedCursor.X,SavedCursor.Y);
  CharAttr := SavedAttr;
  CharAttr2 := SavedAttr2;
  SetupDC;
end;

procedure VTWindow.ParseEscape(b: byte); {b is final char}

  procedure ESCSharp(b: byte);
  var
    TmpPtr: LongInt;
    i: integer;
  begin
    case b of
      Ord('8'):  {Fill screen with "E"}
        begin
          UpdateStr;
          TmpPtr := GetLinePtr(PageStart);
          for i := 0 to NumOfLines-1 do
          begin
            FillChar(CodeBuff[TmpPtr],NumOfColumns,Ord('E'));
            FillChar(AttrBuff[TmpPtr],NumOfColumns,AttrDefault);
            FillChar(AttrBuff2[TmpPtr],NumOfColumns,AttrDefault2);
            TmpPtr := NextLinePtr(TmpPtr);
          end;
          StrChangeCount := 0;
          UpdateRect(DC,WinOrgX,WinOrgY,WinOrgX+WinWidth-1,WinOrgY+WinHeight-1);
          MoveCursor(0,0);
          ParseMode := ModeFirst;
        end;
    end;
  end;

  {select double byte code set}
  procedure DBCSSelect(b: byte);
  var
    Dist: integer;
  begin
    if ts.Language<>IdJapanese then exit;

    case ICount of

      1: if (b=Ord('@')) or (b=Ord('B')) then
      begin
        Gn[0] := IdKanji; {Kanji -> G0}
        Glr[0] := 0;  {G0->GL}
      end;

      2: begin
        {Second intermediate char must be
         '(' or ')' or '*' or '+'. }
        Dist := (IntChar[2]-Ord('(')) and 3; {G0 - G3}
        if (b=Ord('1')) or (b=Ord('3')) or
           (b=Ord('@')) or (b=Ord('B')) then
        begin  
          Gn[Dist] := IdKanji; {Kanji -> G0-3}
          if Dist=0 then Glr[0] := 0;  {G0->GL}
        end;
      end;

    end;
  end;


  procedure ESCPercent(b: byte);
  begin
    case b of
      Ord('0'): if SelectParserFlag and (ts.AutoWinSwitch>0) then
                  ChangeEmu := IdTEK; {enter TEK mode}
    end;
  end;

  {select single byte code set}
  procedure SBCSSelect(b: integer);
  var
    Dist: integer;
  begin
    {Intermediate char must be
     '(' or ')' or '*' or '+'. }
    Dist := (IntChar[1]-Ord('(')) and 3; {G0 - G3}

    case b of
      ord('0'): Gn[Dist] := IdSpecial;
      ord('<'): Gn[Dist] := IdASCII;
      ord('>'): Gn[Dist] := IdASCII;
      ord('B'): Gn[Dist] := IdASCII;
      ord('H'): Gn[Dist] := IdASCII;
      ord('I'): if ts.Language=IdJapanese then
                  Gn[Dist] := IdKatakana;
      ord('J'): Gn[Dist] := IdASCII;
    end;

    if Dist=0 then Glr[0] := 0;  {G0->GL}
  end;

begin

  case Icount of

    {no intermediate char}
    0: case b of
      Ord('7'): SaveCursor;
      Ord('8'): RestoreCursor;
      Ord('='): AppliKeyMode := true;
      Ord('>'): AppliKeyMode := false;
      Ord('D'): LineFeed(0);  {IND}
      Ord('E'): begin                  {NEL}
                  MoveCursor(0,CursorY);
                  LineFeed(0);
                end;
      Ord('H'): SetTabStop;            {HTS}
      Ord('M'): CursorUpWithScroll;    {RI}
      Ord('N'): begin                  {SS2}
                  GLtmp := 2;
                  SSflag := true;
                end;
      Ord('O'): begin                  {SS3}
                  GLtmp := 3;
                  SSflag := true;
                end;
      Ord('P'): begin                  {DCS}
                  SavedMode := ParseMode;
                  ESCFlag := false;
                  NParam := 1;
                  Param[1] := -1;
                  ParseMode := ModeDCS;
                  exit;
                end;
      Ord('X'): begin                  {SOS}
                  SavedMode := ParseMode;
                  ESCFlag := false;
                  ParseMode := ModeSOS;
                  exit;
                end;
      Ord('Z'): AnswerTerminalType;
      Ord('['): begin                  {CSI}
                  ICount := 0;
                  FirstPrm := true;
                  NParam := 1;
                  Param[1] := -1;
                  Prv := 0;
                  ParseMode := ModeCSI;
                  exit;
                end;
      Ord('\'): ;                      {ST}
      Ord(']'): begin  {XTERM sequence (OSC)}
                  NParam := 1;
                  Param[1] := 0;
                  ParseMode := ModeXS;
                  exit;  
                end;
      Ord('^')..Ord('_'): begin  {PM, APC}
                  SavedMode := ParseMode;
                  ESCFlag := false;
                  ParseMode := ModeSOS;
                  exit;
                end;
      Ord('c'): begin; {Hardware reset}
                  ResetTerminal;
                  ClearUserKey;
                  ClearBuffer;
                  if ts.PortType=IdSerial then
                  begin
                    { reset serial port }
                    CommResetSerial(@ts,@cv);
                  end;
                  NewLine(PageStart);
                end;
      Ord('n'): Glr[0] := 2; {LS2}
      Ord('o'): Glr[0] := 3; {LS3}               
      Ord('|'): Glr[1] := 3; {LS3R}
      Ord('}'): Glr[1] := 2; {LS2R}
      Ord('~'): Glr[1] := 1; {LS1R}
    end; {of case Icount=0}

    {one intermediate char}
    1: case IntChar[1] of
      Ord('#'): ESCSharp(b);
      Ord('$'): DBCSSelect(b);
      Ord('%'): ESCPercent(b);
      Ord('(')..Ord('+'): SBCSSelect(b);
    end;

    {two intermediate char}
    2: if (IntChar[1]=Ord('$')) and
          (Ord('(')<=IntChar[2]) and
          (IntChar[2]<=Ord('+')) then DBCSSelect(b);

  end;
  ParseMode := ModeFirst;

end;

procedure VTWindow.ParseCS(b: byte); {b is final char}

  procedure InsertCharacter;
  var
    count: integer;
  begin
    UpdateStr;
    if Param[1]<1 then Param[1] := 1;
    count := Param[1];

    if ts.Language=IdJapanese then
      EraseKanji(1); {if cursor is on right half of a kanji, erase the kanji}

    if count > NumOfColumns - CursorX then count := NumOfColumns - CursorX;
    Move(CodeLine[CursorX],CodeLine[CursorX+Count],
         NumOfColumns-Count-CursorX);
    Move(AttrLine[CursorX],AttrLine[CursorX+Count],
         NumOfColumns-Count-CursorX);
    Move(AttrLine2[CursorX],AttrLine2[CursorX+Count],
         NumOfColumns-Count-CursorX);
    FillChar(CodeLine[CursorX],Count,$20);
    FillChar(AttrLine[CursorX],Count,AttrDefault);
    FillChar(AttrLine2[CursorX],Count,AttrDefault2);
    {last char in current line is kanji first?}
    if (byte(AttrLine[NumOfColumns-1]) and AttrKanji) <> 0 then
    begin
      {then delete it}
      CodeLine[NumOfColumns-1] := #$20;
      AttrLine[NumOfColumns-1] := char(AttrDefault);
      AttrLine2[NumOfColumns-1] := char(AttrDefault2);
    end;
    UpdateRect(DC,CursorX,CursorY,NumOfColumns-1,CursorY);
  end;

  procedure CursorUp;
  begin
    if Param[1]<1 then Param[1] := 1;

    if CursorY >= CursorTop then
    begin
      if CursorY-Param[1] > CursorTop then
        MoveCursor(CursorX,CursorY-Param[1])
      else
        MoveCursor(CursorX,CursorTop);
    end
    else begin
      if CursorY > 0 then
        MoveCursor(CursorX,CursorY-Param[1])
      else
        MoveCursor(CursorX,0);
    end;
  end;

  procedure CursorUp1;
  begin
    MoveCursor(0,CursorY);
    CursorUp;
  end;

  procedure CursorDown;
  begin
    if Param[1]<1 then Param[1] := 1;

    if CursorY <= CursorBottom then
    begin
      if CursorY+Param[1] < CursorBottom then
        MoveCursor(CursorX,CursorY+Param[1])
      else
        MoveCursor(CursorX,CursorBottom);
    end
    else begin
      if CursorY < NumOfLines-1 then
        MoveCursor(CursorX,CursorY+Param[1])
      else
        MoveCursor(CursorX,NumOfLines);
    end;
  end;

  procedure CursorDown1;
  begin
    MoveCursor(0,CursorY);
    CursorDown;
  end;

  procedure ScreenErase;
  var
    TmpPtr: longint;
    offset: integer;
    i: integer;
  begin
    if Param[1] = -1 then Param[1] := 0;
    UpdateStr;
    R.left := 0;
    R.right := ScreenWidth;
    case Param[1] of
      0:
        begin
          if ts.Language=IdJapanese then
            EraseKanji(1); {if cursor is on right half of a kanji, erase the kanji}
          offset := CursorX;
          TmpPtr := GetLinePtr(PageStart+CursorY);
          for i := CursorY to NumOfLines-1 do
          begin
            FillChar(CodeBuff[TmpPtr+offset],NumOfColumns-offset,$20);
            FillChar(AttrBuff[TmpPtr+offset],NumOfColumns-offset,AttrDefault);
            FillChar(AttrBuff2[TmpPtr+offset],NumOfColumns-offset,AttrDefault2);
            offset := 0;
            TmpPtr := NextLinePtr(TmpPtr);
          end;
          R.top := (CursorY+1-WinOrgY)*FontHeight;
          R.bottom := ScreenHeight;
          FillRect(DC,R,BackGround);
          R.left := (CursorX-WinOrgX)*FontWidth;
          R.bottom := R.top;
          R.top := R.bottom-FontHeight;
          FillRect(DC,R,BackGround);
        end;
      1:
        begin
          if ts.Language=IdJapanese then
            EraseKanji(0); {if cursor is on left half of a kanji, erase the kanji}
          offset := NumOfColumns;
          TmpPtr := GetLinePtr(PageStart);
          for i := 0 to CursorY do
          begin
	    if i=CursorY then offset := CursorX+1;
            FillChar(CodeBuff[TmpPtr],offset,$20);
            FillChar(AttrBuff[TmpPtr],offset,AttrDefault);
            FillChar(AttrBuff2[TmpPtr],offset,AttrDefault2);
            TmpPtr := NextLinePtr(TmpPtr);
          end;
          R.top := -WinOrgY*FontHeight;
          R.bottom := (CursorY-WinOrgY)*FontHeight;
          FillRect(DC,R,BackGround);
          R.top := R.bottom;
          R.bottom := R.top + FontHeight;
          R.right := (CursorX+1-WinOrgX)*FontWidth;
          FillRect(DC,R,BackGround);
        end;
      2:
        begin
          BuffScroll(NumOfLines);
          WinOrgY := WinHeight - NumOfLines;
          NewOrgY := WinHeight - NumOfLines;
          ScrollCount := 0;
          if (BuffEnd=WinHeight) and (ts.EnableScrollBuff>0) then
          begin
            Scroller^.SetRange(Scroller^.XRange,1);
            Scroller^.YRange := 0;
          end
          else
            Scroller^.SetRange(Scroller^.XRange,BuffEnd-WinHeight);
          Scroller^.YPos := Scroller^.YPos - NumOfLines;
          Scroller^.ScrollTo(Scroller^.XPos,PageStart);
          NewLine(PageStart+CursorY);
        end;
    end;
  end;

  procedure InsertLine;
  var
    i, count, YEnd: integer;
    SrcPtr, DestPtr: longint;
  begin
    if CursorY < CursorTop then exit;
    if CursorY > CursorBottom then exit;
    if Param[1]<1 then Param[1] := 1;
    UpdateStr;
    count := Param[1];
    YEnd := CursorBottom;
    if CursorY > YEnd then YEnd := NumOfLines-1;
    if count > YEnd+1 - CursorY then count := YEnd+1 - CursorY;
    SrcPtr := GetLinePtr(PageStart+YEnd-Count);
    DestPtr := GetLinePtr(PageStart+YEnd);
    for i:= YEnd-Count downto CursorY do
    begin
      Move(CodeBuff[SrcPtr],CodeBuff[DestPtr],NumOfColumns);
      Move(AttrBuff[SrcPtr],AttrBuff[DestPtr],NumOfColumns);
      Move(AttrBuff2[SrcPtr],AttrBuff2[DestPtr],NumOfColumns);
      SrcPtr := PrevLinePtr(SrcPtr);
      DestPtr := PrevLinePtr(DestPtr);
    end;
    for i := 1 to Count do
    begin
      FillChar(CodeBuff[DestPtr],NumOfColumns,$20);
      FillChar(AttrBuff[DestPtr],NumOfColumns,AttrDefault);
      FillChar(AttrBuff2[DestPtr],NumOfColumns,AttrDefault2);
      DestPtr := PrevLinePtr(DestPtr);
    end;

    if Active and CompletelyVisible and (CursorY >= WinOrgY) then
    begin
      R.left := 0;
      R.right := ScreenWidth;
      R.top := (CursorY-WinOrgY)*FontHeight;
      R.bottom := (YEnd+1-WinOrgY)*FontHeight;
      ScrollWindowEx(HWindow,0,FontHeight*Count,@R,@R,0,nil,0);
      R.bottom := R.top + FontHeight*Count;
      FillRect(DC,R,BackGround);
    end
    else
      UpdateRect(DC,WinOrgX,CursorY,WinOrgX+WinWidth-1,YEnd);

  end;

  procedure LineErase;
  begin
    if Param[1] = -1 then Param[1] := 0;
    UpdateStr;
    R.Top := (CursorY-WinOrgY)*FontHeight;
    R.Bottom := R.top+FontHeight;
    case Param[1] of
      {erase char from cursor to end of line}
      0:
        begin
          FillChar(CodeLine[CursorX],NumOfColumns-CursorX,$20);
          FillChar(AttrLine[CursorX],NumOfColumns-CursorX,AttrDefault);
          FillChar(AttrLine2[CursorX],NumOfColumns-CursorX,AttrDefault2);

          R.left := (CursorX-WinOrgX)*FontWidth;
          R.right := ScreenWidth;
          FillRect(DC,R,BackGround);
        end;
      {erase char from start of line to cursor}
      1:
        begin
          FillChar(CodeLine[0],CursorX+1,$20);
          FillChar(AttrLine[0],CursorX+1,AttrDefault);
          FillChar(AttrLine2[0],CursorX+1,AttrDefault2);
          R.left := 0;
          R.right := (CursorX+1-WinOrgX)*FontWidth;
          FillRect(DC,R,BackGround);
        end;
      {erase entire line}
      2:
        begin
          FillChar(CodeLine[0],NumOfColumns,$20);
          FillChar(AttrBuff[0],NumOfColumns,AttrDefault);
          FillChar(AttrBuff2[0],NumOfColumns,AttrDefault2);
          R.left := 0;
          R.right := ScreenWidth;
          FillRect(DC,R,BackGround);
        end;
    end;
  end;

  procedure DeleteNLines;
  var
    i, YEnd: integer;
    SrcPtr, DestPtr: longint;
  begin
    if CursorY < CursorTop then exit;
    if CursorY > CursorBottom then exit;
    if Param[1]<1 then Param[1] := 1;
    UpdateStr;

    YEnd := CursorBottom;
    if CursorY > YEnd then YEnd := NumOfLines-1;

    if Param[1] > YEnd+1-CursorY then Param[1] := YEnd+1-CursorY;
    SrcPtr := GetLinePtr(PageStart+CursorY+Param[1]);
    DestPtr := GetLinePtr(PageStart+CursorY);
    for i:=CursorY to YEnd-Param[1] do
    begin
      Move(CodeBuff[SrcPtr],CodeBuff[DestPtr],NumOfColumns);
      Move(AttrBuff[SrcPtr],AttrBuff[DestPtr],NumOfColumns);
      Move(AttrBuff2[SrcPtr],AttrBuff2[DestPtr],NumOfColumns);
      SrcPtr := NextLinePtr(SrcPtr);
      DestPtr := NextLinePtr(DestPtr);
    end;
    for i := YEnd+1-Param[1] to YEnd do
    begin
      FillChar(CodeBuff[DestPtr],NumOfColumns,$20);
      FillChar(AttrBuff[DestPtr],NumOfColumns,AttrDefault);
      FillChar(AttrBuff2[DestPtr],NumOfColumns,AttrDefault2);
      DestPtr := NextLinePtr(DestPtr);
    end;
   
    if Active and CompletelyVisible and (YEnd+1-WinOrgY <= WinHeight) then
    begin
      R.left := 0;
      R.right := ScreenWidth;
      R.top := (CursorY-WinOrgY)*FontHeight;
      R.bottom := (YEnd+1-WinOrgY)*FontHeight;
      ScrollWindowEx(HWindow,0,-FontHeight*Param[1],@R,@R,0,nil,0);
      R.top := R.bottom - FontHeight*Param[1];
      FillRect(DC,R,BackGround);
    end
    else
      UpdateRect(DC,WinOrgX,CursorY,WinOrgX+WinWidth-1,YEnd);

  end;

  procedure DeleteCharacter;
  var
    Count: integer;
  begin
    if Param[1]<1 then Param[1] := 1;
    UpdateStr;
    Count := Param[1];

    if ts.Language=IdJapanese then
    begin
      EraseKanji(0); {if cursor is on left harf of a kanji, erase the kanji}
      EraseKanji(1); {if cursor on right half...}
    end;

    if Count > NumOfColumns-CursorX then Count := NumOfColumns-CursorX;
    Move(CodeLine[CursorX+Count],
         CodeLine[CursorX],NumOfColumns-Count-CursorX);
    Move(AttrLine[CursorX+Count],
         AttrLine[CursorX],NumOfColumns-Count-CursorX);
    Move(AttrLine2[CursorX+Count],
         AttrLine2[CursorX],NumOfColumns-Count-CursorX);
    FillChar(CodeLine[NumOfColumns-Count],Count,$20);
    FillChar(AttrLine[NumOfColumns-Count],Count,AttrDefault);
    FillChar(AttrLine2[NumOfColumns-Count],Count,AttrDefault2);

    UpdateRect(DC,CursorX,CursorY,WinOrgX+WinWidth-1,CurSorY);

  end;

  procedure EraseCharacter;
  var
    Count: integer;
  begin
    if Param[1]<1 then Param[1] := 1;
    UpdateStr;
    Count := Param[1];

    if ts.Language=IdJapanese then
    begin
      EraseKanji(0); {if cursor is on left harf of a kanji, erase the kanji}
      EraseKanji(1); {if cursor on right half...}
    end;

    if Count > NumOfColumns-CursorX then Count := NumOfColumns-CursorX;
    FillChar(CodeLine[CursorX],Count,$20);
    FillChar(AttrLine[CursorX],Count,AttrDefault);
    FillChar(AttrLine2[CursorX],Count,AttrDefault2);

    R.left := (CursorX-WinOrgX)*FontWidth;
    R.right := R.left + Count*FontWidth;
    R.top := (CursorY-WinOrgY)*FontHeight;
    R.bottom := R.top + FontHeight;
    FillRect(DC,R,BackGround);
  end;

  procedure MoveToColumnN;
  begin
    if Param[1]<1 then Param[1] := 1;
    dec(Param[1]);
    if Param[1] < 0 then Param[1] := 0;
    if Param[1] > NumOfColumns-1 then Param[1] := NumOfColumns-1;
    MoveCursor(Param[1],CursorY);
  end;

  procedure CursorRight;
  begin
    if Param[1]<1 then Param[1] := 1;
    if CursorX + Param[1] > NumOfColumns-1 then
      MoveCursor(NumOfColumns-1,CursorY)
    else
      MoveCursor(CursorX+Param[1],CursorY);
  end;

  procedure CursorLeft;
  begin
    if Param[1]<1 then Param[1] := 1;
    if CursorX-Param[1] < 0 then
      MoveCursor(0,CursorY)
    else
      MoveCursor(CursorX-Param[1],CursorY);
  end;

  procedure MoveToLineN;
  begin
    if Param[1]<1 then Param[1] := 1;
    if RelativeOrgMode
    then begin
      if CursorTop+Param[1]-1 > CursorBottom then
        MoveCursor(CursorX,CursorBottom)
      else
        MoveCursor(CursorX,CursorTop+Param[1]-1);
    end
    else begin
      if Param[1] > NumOfLines then
        MoveCursor(CursorX,NumOfLines-1)
      else
        MoveCursor(CursorX,Param[1]-1);
    end;
  end;

  procedure MoveToXY;
  var
    NewX, NewY: integer;
  begin
    if Param[1]<1 then Param[1] := 1;
    if (NParam < 2) or (Param[2]<1) then Param[2] := 1;
    NewX := Param[2] - 1;
    if NewX > NumOfColumns-1 then NewX := NumOfColumns-1;
    if RelativeOrgMode
    then begin
      NewY := CursorTop + Param[1] - 1;
      if NewY > CursorBottom then NewY := CursorBottom;
    end
    else begin
      NewY := Param[1] - 1;
      if NewY > NumOfLines-1 then NewY := NumOfLines-1;
    end;
    MoveCursor(NewX,NewY);
  end;

  procedure DeleteTabStop;
  var
    i,j: integer;
  begin
    if Param[1]=-1 then Param[1] := 0;
    if NTabStops>0 then
      case Param[1] of
        0: begin
             i := 0;
             while (TabStops[i]<>CursorX) and (i<NTabStops-1) do
               inc(i);
             if TabStops[i] = CursorX then
             begin
               Dec(NTabStops);
               for j:=i to NTabStops do
                 TabStops[j] := TabStops[j+1];
             end;
           end;
        3: NTabStops := 0;
      end;
  end;

  procedure h_Mode;
  begin
    case Param[1] of
      4: InsertMode := true;
      20: begin
            LFMode := true;
            ts.CRSend := IdCRLF;
            cv.CRSend := IdCRLF;
          end;
      38: if ts.AutoWinSwitch>0 then ChangeEmu := IdTEK; {Enter TEK Mode}
      59: if ts.Language=IdJapanese then
          begin {kanji terminal}
            Gn[0] := IdASCII;
            Gn[1] := IdKatakana;
            Gn[2] := IdKatakana;
            Gn[3] := IdKanji;
            Glr[0] := 0;
            Glr[1] := 3;
          end;
    end;
  end;

  procedure i_Mode;
  begin
    if Param[1]=-1 then Param[1] := 0;
    case Param[1] of
      {print screen}
      0: Print(true);
      {printer controller mode off}
      4: ; {See 'teraprn.pas'}
      {printer controller mode on}
      5: begin
          if not AutoPrintMode then
            OpenPrnFile;
          PrinterMode := true;
          PrnParseMode := IdPrnNormal;
        end;
    end;
  end;

  procedure l_Mode;
  begin
    case Param[1] of
      4: InsertMode := false;
      20: begin
            LFMode := false;
            ts.CRSend := IdCR;
            cv.CRSend := IdCR;
          end;
      59: if ts.Language=IdJapanese then
          begin {katakana terminal}
            Gn[0] := IdASCII;
            Gn[1] := IdKatakana;
            Gn[2] := IdKatakana;
            Gn[3] := IdKanji;
            Glr[0] := 0;
            Glr[1] := 3;
          end;
    end;
  end;

  procedure n_Mode;
  var
    Report: array[0..15] of char;
    NumStr: string[4];
  begin
    case Param[1] of
      5: CommBinaryOut(@cv,#$1B'[0n',4); {Device Status Report -> Ready}
      6: begin
           {Cursor Position Report}
           StrCopy(Report,#$1B'[');
           Str(CursorY+1,NumStr);
           StrPCopy(StrEnd(Report),NumStr);
           StrCat(Report,';');
           Str(CursorX+1,NumStr);
           StrPCopy(StrEnd(Report),NumStr);
           StrCat(Report,'R');
           CommBinaryOut(@cv,Report,StrLen(Report));
         end;
    end;
  end;

  procedure SetAttr;
  var
    i, P: integer;
  begin
    UpdateStr;
    for i:=1 to NParam do
    begin
      P := Param[i];
      if P<0 then P := 0;
      case P of
        {Clear}
        0: begin
            CharAttr := AttrDefault;
            CharAttr2 := AttrDefault2;
          end;
        {Bold}
        1: CharAttr := CharAttr or AttrBold;
        {Under line}
        4: CharAttr := CharAttr or AttrUnder;
        {Blink}
        5: CharAttr := CharAttr or AttrBlink;
        {Reverse}
        7: CharAttr := CharAttr or AttrReverse;
        {Bold off}
        22: CharAttr := CharAttr and not AttrBold;
        {Under line off}
        24: CharAttr := CharAttr and not AttrUnder;
        {Blink off}
        25: CharAttr := CharAttr and not AttrBlink;
        {Reverse off}
        27: CharAttr := CharAttr and not AttrReverse;
        {Text color}
        30..37: CharAttr2 := CharAttr2 and (Attr2Back or Attr2BackMask)
                             or (P-30) or Attr2Fore;
        {Back color}
        40..47: CharAttr2 := CharAttr2 and (Attr2Fore or Attr2ForeMask)
                             or ((P-40) shl SftAttrBack) or Attr2Back;
        {Reset color attributes}
        100: CharAttr2 := AttrDefault2;
      end;
    end;
    SetupDC;
  end;

  procedure SetScrollRegion;
  begin
    if Param[1]<1 then Param[1] :=1;    
    if (NParam < 2) or (Param[2]<1) then Param[2] := NumOfLines;
    Dec(Param[1]);
    Dec(Param[2]);
    if Param[1] > NumOfLines-1 then Param[1] := NumOfLines-1;
    if Param[2] > NumOfLines-1 then Param[2] := NumOfLines-1;
    if Param[1] >= Param[2] then exit;
    CursorTop := Param[1];
    CursorBottom := Param[2];
    if RelativeOrgMode then MoveCursor(0,CursorTop)
                       else MoveCursor(0,0);
  end;

  procedure SunSequence; {Sun terminal private sequences}
  var
    Report: array[0..15] of char;
    NumStr: string[4];
  begin
    case Param[1] of
      8: begin {set terminal size}
        if (Param[2]<=1) or (NParam<2) then Param[2] := 24;
        if (Param[3]<=1) or (NParam<3) then Param[3] := 80;
        if (Param[2]=NumOfLines) and (Param[3]=NumOfColumns) then exit;
        UpdateStr;
        ChangeTerminalSize(Param[3],Param[2]);
        NewLine(PageStart);
      end;
      14: begin {get window size???}
          {Of cource, this is not actual window size} 
          CommBinaryOut(@cv,#$1B'[4;640;480t',12);
        end;
      18: begin {get terminal size}
        {Cursor Position Report}
        StrCopy(Report,#$1B'[8;');
        Str(NumOfLines,NumStr);
        StrPCopy(StrEnd(Report),NumStr);
        StrCat(Report,';');
        Str(NumOfColumns,NumStr);
        StrPCopy(StrEnd(Report),NumStr);
        StrCat(Report,'t');
        CommBinaryOut(@cv,Report,StrLen(Report));
      end;
    end;
  end;

  procedure CSGT(b: byte);
  begin
    case b of
      Ord('c'): {second terminal report}
        CommBinaryOut(@cv,#$1B'[>32;10;2c',11);
    end;
  end;

  procedure CSQuest(b: byte);

    procedure ExchangeColor;
    var
      ColorRef: TColorRef;
    begin
      UpdateStr;

      ColorRef := ts.VTColor[0];
      ts.VTColor[0] := ts.VTColor[1];
      ts.VTColor[1] := ColorRef;
      ReleaseDC(HWindow,DC);
      DeleteObject(Background);
      BackGround := CreateSolidBrush(ts.VTColor[1]);
      DC := GetDC(HWindow);
      SetupDC;
      UpdateRect(DC,WinOrgX,WinOrgY,WinOrgX+WinWidth-1,WinOrgY+WinHeight-1);
    end;

    procedure qh_Mode;
    var
      i: integer;
    begin
      for i := 1 to NParam do
        case Param[i] of
          1: AppliCursorMode := true;
          3: if NumOfColumns < 132 then
            begin
              UpdateStr;
              ChangeTerminalSize(132,NumOfLines);
              NewLine(PageStart);
            end;
          5: begin
            if ReverseColor then exit;
              ReverseColor := true;
              {Exchange text/back color}
              ExchangeColor;
            end;
          6: begin
            RelativeOrgMode := true;
            MoveCursor(0,CursorTop);
          end;
          7: AutoWrapMode := true;
          8: AutoRepeatMode := true;
          19: PrintEX := true;
        end;
    end;

    procedure qi_Mode;
    begin
      if Param[1]=-1 then Param[1] := 0;
      case Param[1] of
        1: begin
          if HPrnFile<=0 then
            OpenPrnFile;
          PrintCurrentLine(LF);
          if not AutoPrintMode then
            ClosePrnFile;
        end;
        {auto print mode off}
        4: if AutoPrintMode then
          begin
            ClosePrnFile;
            AutoPrintMode := false;
          end;
        {auto print mode on}
        5: if not AutoPrintMode then
          begin
            OpenPrnFile;
            AutoPrintMode := true;
          end;
      end;
    end;

    procedure ql_Mode;
    var
      i: integer;
    begin
      for i := 1 to NParam do
        case Param[i] of
          1: AppliCursorMode := false;
          3: begin
            UpdateStr;
            ChangeTerminalSize(80,NumOfLines);
            NewLine(PageStart);
          end;
          5: begin
            if not ReverseColor then exit;
            ReverseColor := false;
            {Exchange text/back color}
            ExchangeColor;
          end;
          6: begin
            RelativeOrgMode := false;
            MoveCursor(0,0);
          end;
          7: AutoWrapMode := false;
          8: AutoRepeatMode := false;
          19: PrintEX := false;
        end;
    end;

    procedure qn_Mode;
    begin
    end;

  begin
    case b of
      Ord('h'): qh_Mode;
      Ord('i'): qi_Mode;
      Ord('l'): ql_Mode;
      Ord('n'): qn_Mode;
    end;
  end;

  procedure CSExc(b: byte);
  begin
    case b of
      Ord('p'): begin
        {Software reset}
        UpdateStr;
        ResetTerminal;
        Scroller^.ScrollTo(0,PageStart);
        NewLine(PageStart);
      end;
    end;
  end;

begin

  case Icount of

    {no intermediate char}
    0: case Prv of

      {no private parameters}
      0: case b of
        Ord('@'): InsertCharacter;
        Ord('A'): CursorUp;
        Ord('B'): CursorDown;
        Ord('C'): CursorRight;
        Ord('D'): CursorLeft;
        Ord('E'): CursorDown1;
        Ord('F'): CursorUp1;
        Ord('G'): MoveToColumnN;
        Ord('H'): MoveToXY;
        Ord('J'): ScreenErase;
        Ord('K'): LineErase;
        Ord('L'): InsertLine;
        Ord('M'): DeleteNLines;
        Ord('P'): DeleteCharacter;
        Ord('X'): EraseCharacter;
        Ord('`'): MoveToColumnN;
        Ord('a'): CursorRight;
        Ord('c'): AnswerTerminalType;
        Ord('d'): MoveToLineN;
        Ord('e'): CursorUp;
        Ord('f'): MoveToXY;
        Ord('g'): DeleteTabStop;
        Ord('h'): h_Mode;
        Ord('i'): i_Mode;
        Ord('l'): l_Mode;
        Ord('m'): SetAttr;
        Ord('n'): n_Mode;
        Ord('r'): SetScrollRegion;
        Ord('s'): SaveCursor;
        Ord('t'): SunSequence;
        Ord('u'): RestoreCursor;
      end; {of case Prv=0}

      {private parameter = '>'}
      Ord('>'): CSGt(b);
      {private parameter = '?'}
      Ord('?'): CSQuest(b);

    end;

    {one intermediate char}
    1: case IntChar[1] of
      {intermediate char = '!'}
      Ord('!'): CSExc(b);
    end;

  end; {of case Icount}

  ParseMode := ModeFirst;
end;


procedure VTWindow.CursorUpWithScroll;

  procedure ScrollUp1Line;
  var
    i: integer;
    SrcPtr, DestPtr: longint;
  begin
    UpdateStr;
    if (CursorTop<=CursorY) and (CursorY<=CursorBottom) then
    begin
      DestPtr := GetLinePtr(PageStart+CursorBottom);
      for i:= CursorBottom-1 downto CursorTop do
      begin
        SrcPtr := PrevLinePtr(DestPtr);
        Move(CodeBuff[SrcPtr],CodeBuff[DestPtr],NumOfColumns);
        Move(AttrBuff[SrcPtr],AttrBuff[DestPtr],NumOfColumns);
        Move(AttrBuff2[SrcPtr],AttrBuff2[DestPtr],NumOfColumns);
        DestPtr := SrcPtr;
      end;
      FillChar(CodeBuff[SrcPtr],NumOfColumns,$20);
      FillChar(AttrBuff[SrcPtr],NumOfColumns,AttrDefault);
      FillChar(AttrBuff2[SrcPtr],NumOfColumns,AttrDefault2);

      if Active and CompletelyVisible and (CursorTop >= WinOrgY) then
      begin
        R.left := 0;
        R.right := ScreenWidth;
        R.top := (CursorTop-WinOrgY)*FontHeight;
        R.bottom := (CursorBottom+1-WinOrgY)*FontHeight;
        ScrollWindowEx(HWindow,0,FontHeight,@R,@R,0,nil,0);
        R.bottom := R.top + FontHeight;
        FillRect(DC,R,BackGround);
      end
      else
        UpdateRect(DC,WinOrgX,CursorTop,WinOrgX+WinWidth-1,CursorBottom);

    end;
  end;

begin
  if ((0<CursorY) and (CursorY<CursorTop)) or
     (CursorTop<CursorY) then
    MoveCursor(CursorX,CursorY-1)
  else if CursorY=CursorTop then
    ScrollUp1Line;
end;

procedure VTWindow.SetTabStop;
var
  i,j: integer;
begin
  if NTabStops<NumOfColumns then
  begin
    i := 0;
    while (Tabstops[i]<CursorX) and (i<NTabstops) do
      inc(i);
    
    if (i<NTabstops) and (Tabstops[i]=CursorX) then exit;

    for j:=NTabStops downto i+1 do
      TabStops[j] := TabStops[j-1];
    Tabstops[i] := CursorX;
    inc(NTabstops);
  end;
end;

procedure VTWindow.AnswerTerminalType;
var
  Tmp: array[0..30] of char;
begin
  if ts.TerminalID<IdVT320 then
    StrCopy(Tmp,#$1b'[?')
  else
    StrCopy(Tmp,#$9B'?');

  case ts.TerminalID of
    IdVT100:  StrCat(Tmp,'1;2');
    IdVT100J: StrCat(Tmp,'5;2');
    IdVT101:  StrCat(Tmp,'1;0');
    IdVT102:  StrCat(Tmp,'6');
    IdVT102J: StrCat(Tmp,'15');
    IdVT220J: StrCat(Tmp,'62;1;2;5;6;7;8');
    IdVT282:  StrCat(Tmp,'62;1;2;4;5;6;7;8;10;11');
    IdVT320:  StrCat(Tmp,'63;1;2;6;7;8');
    IdVT382:  StrCat(Tmp,'63;1;2;4;5;6;7;8;10;15');
  end;
  StrCat(Tmp,'c');

  CommBinaryOut(@cv,Tmp,StrLen(Tmp)); {Report terminal ID}
end;

procedure VTWindow.XSequence(b: byte);
begin
  if NParam=1 then
    case b of
      $30..$39: if Param[1]<1000 then
                  Param[1] := Param[1]*10 + b - $30;
    else
      NParam := 2;
    end
  else
    case b of
      NUL..US: begin
          if Param[1]<=2 then
          begin
            ts.Title[NParam-2] := #0;
            ChangeTitle;
          end;
          ParseMode := ModeFirst;
        end;
    else
      if (Param[1]<=2) and (NParam-2<SizeOf(ts.Title)-1) then
      begin
        ts.Title[NParam-2] := char(b);
        inc(NParam);
      end;
    end;
end;

procedure VTWindow.DLESeen(b: byte);
begin
  ParseMode := ModeFirst;
  if ts.BPAuto and (b=ord('B')) then
    BPStart(IdBPAuto); {Auto B-Plus activation}
  ChangeEmu := -1;
end;

procedure VTWindow.CANSeen(b: byte);
begin
  ParseMode := ModeFirst;
  if ts.ZmodemAuto and (b=ord('B')) then
    ZMODEMStart(IdZAuto); {Auto ZMODEM activation}
  ChangeEmu := -1;
end;
